/* Generated by IDL compiler version OSF DCE T1.1.0-03 */
#ifdef VMS
#pragma nostandard
#endif
#include <srvsvc.h>
#include <dce/idlddefs.h>

static rpc_syntax_id_t IDL_transfer_syntaxes[1] = {
{
{0x8a885d04u, 0x1ceb, 0x11c9, 0x9f, 0xe8, {0x8, 0x0, 0x2b, 0x10, 0x48, 0x60}},
2}};

static rpc_if_rep_t IDL_ifspec = {
  1, /* ifspec rep version */
  42, /* op count */
  3, /* if version */
  {0x4b324fc8u, 0x1670, 0x01d3, 0x12, 0x78, {0x5a, 0x47, 0xbf, 0x6e, 0xe1, 0x88}},
  2, /* stub/rt if version */
  {0, NULL}, /* endpoint vector */
  {1, IDL_transfer_syntaxes} /* syntax vector */
,NULL /* server_epv */
,NULL /* manager epv */
};
/* global */ rpc_if_handle_t srvsvc_v3_0_c_ifspec = (rpc_if_handle_t)&IDL_ifspec;
static idl_ulong_int IDL_offset_vec[] = {
0,	/* sentinel */
		/* struct SHARE_ENUM_STRUCT size and offsets */
/*      1 */ sizeof(SHARE_ENUM_STRUCT )	/* sizeof SHARE_ENUM_STRUCT */,
/*      2 */ offsetof(SHARE_ENUM_STRUCT , share_union),	/* field share_union offset */
		/* union SHARE_ENUM_UNION size and offset */
/*      3 */ sizeof(SHARE_ENUM_UNION )	/* sizeof SHARE_ENUM_UNION */,
/*      4 */ offsetof(SHARE_ENUM_UNION , tagged_union),
		/* struct SHARE_INFO_1_CONTAINER size and offsets */
/*      5 */ sizeof(SHARE_INFO_1_CONTAINER )	/* sizeof SHARE_INFO_1_CONTAINER */,
/*      6 */ offsetof(SHARE_INFO_1_CONTAINER , share_count),	/* field share_count offset */
/*      7 */ offsetof(SHARE_INFO_1_CONTAINER , shares),	/* field shares offset */
		/* struct SHARE_INFO_1 size and offsets */
/*      8 */ sizeof(SHARE_INFO_1 )	/* sizeof SHARE_INFO_1 */,
/*      9 */ offsetof(SHARE_INFO_1 , shi1_share),	/* field shi1_share offset */
/*     10 */ offsetof(SHARE_INFO_1 , shi1_type),	/* field shi1_type offset */
/*     11 */ offsetof(SHARE_INFO_1 , shi1_remark),	/* field shi1_remark offset */
0	/* sentinel */
};

static IDL_rtn_func_t IDL_rtn_vec[] = {
(IDL_rtn_func_t)NULL,	/* sentinel */
(IDL_rtn_func_t)NULL	/* sentinel */
};

static idl_byte IDL_type_vec[] = {
/*      0 */ 0xff,0xff,0xff,0xff,
/*      4 */ 0,		/* big endian */
/*      5 */ 0,		/* ASCII */
/*      6 */ 0xff,0xff,
/*      8 */ 0x00,0x03,	/* interpreter encoding major version 3 */
/*     10 */ 0x00,0x02,	/* interpreter encoding minor version 2 */
/*     12 */ 0x00,0x03,	/* interface srvsvc major version 3 */
/*     14 */ 0x00,0x00,	/* interface srvsvc minor version 0 */
/*     16 */ 0x4b,0x32,0x4f,0xc8,	/* uuid time_low */
/*     20 */ 0x16,0x70,	/* uuid time_mid */
/*     22 */ 0x01,0xd3,	/* uuid time_hi_and_version */
/*     24 */ 0x12,		/* uuid clock_seq_hi_and_reserved */
/*     25 */ 0x78,		/* uuid clock_seq_low */
/*     26 */ 0x5a,0x47,0xbf,0x6e,0xe1,0x88,	/* uuid node */
/*     32 */ 0x00,0x00,0x00,0x00,	/* no storage information */
/*     36 */ 0x00,0x00,0x00,0x04,	/* number of bug flags = 4 */
/*     40 */ 0x00,0x00,0x07,0x9c,	/* index of bug flags = 1948 */
/*     44 */ 0xff,0xff,0xff,0xff,
/*     48 */ 0xff,0xff,0xff,0xff,
/*     52 */ 0xff,0xff,0xff,0xff,
/*     56 */ 0xff,0xff,0xff,0xff,
/*     60 */ 0x00,0x00,0x00,0x2a,	/* number of operations = 42 */
/*     64 */ 0x00,0x00,0x00,0x00,	/* operation NetrCharDevEnum flags */
/*     68 */ 0x00,0x00,0x00,0x02,	/* number of NetrCharDevEnum params = 2 */
/*     72 */ 0x00,0x00,0x00,0x00,	/* number of NetrCharDevEnum [in]s  = 0 */
/*     76 */ 0xff,0xff,0xff,0xff,
/*     80 */ 0x00,0x00,0x00,0x01,	/* number of NetrCharDevEnum [out]s = 1 */
/*     84 */ 0x00,0x00,0x04,0x30,	/* index  of NetrCharDevEnum [out]s = 1072 */
/*     88 */ 0x00,0x00,0x00,0x00,	/* operation NetrCharDevGetInfo flags */
/*     92 */ 0x00,0x00,0x00,0x02,	/* number of NetrCharDevGetInfo params = 2 */
/*     96 */ 0x00,0x00,0x00,0x00,	/* number of NetrCharDevGetInfo [in]s  = 0 */
/*    100 */ 0xff,0xff,0xff,0xff,
/*    104 */ 0x00,0x00,0x00,0x01,	/* number of NetrCharDevGetInfo [out]s = 1 */
/*    108 */ 0x00,0x00,0x04,0x38,	/* index  of NetrCharDevGetInfo [out]s = 1080 */
/*    112 */ 0x00,0x00,0x00,0x00,	/* operation NetrCharDevControl flags */
/*    116 */ 0x00,0x00,0x00,0x02,	/* number of NetrCharDevControl params = 2 */
/*    120 */ 0x00,0x00,0x00,0x00,	/* number of NetrCharDevControl [in]s  = 0 */
/*    124 */ 0xff,0xff,0xff,0xff,
/*    128 */ 0x00,0x00,0x00,0x01,	/* number of NetrCharDevControl [out]s = 1 */
/*    132 */ 0x00,0x00,0x04,0x40,	/* index  of NetrCharDevControl [out]s = 1088 */
/*    136 */ 0x00,0x00,0x00,0x00,	/* operation NetrCharDevQEnum flags */
/*    140 */ 0x00,0x00,0x00,0x02,	/* number of NetrCharDevQEnum params = 2 */
/*    144 */ 0x00,0x00,0x00,0x00,	/* number of NetrCharDevQEnum [in]s  = 0 */
/*    148 */ 0xff,0xff,0xff,0xff,
/*    152 */ 0x00,0x00,0x00,0x01,	/* number of NetrCharDevQEnum [out]s = 1 */
/*    156 */ 0x00,0x00,0x04,0x48,	/* index  of NetrCharDevQEnum [out]s = 1096 */
/*    160 */ 0x00,0x00,0x00,0x00,	/* operation NetrCharDevQGetInfo flags */
/*    164 */ 0x00,0x00,0x00,0x02,	/* number of NetrCharDevQGetInfo params = 2 */
/*    168 */ 0x00,0x00,0x00,0x00,	/* number of NetrCharDevQGetInfo [in]s  = 0 */
/*    172 */ 0xff,0xff,0xff,0xff,
/*    176 */ 0x00,0x00,0x00,0x01,	/* number of NetrCharDevQGetInfo [out]s = 1 */
/*    180 */ 0x00,0x00,0x04,0x50,	/* index  of NetrCharDevQGetInfo [out]s = 1104 */
/*    184 */ 0x00,0x00,0x00,0x00,	/* operation NetrCharDevQSetInfo flags */
/*    188 */ 0x00,0x00,0x00,0x02,	/* number of NetrCharDevQSetInfo params = 2 */
/*    192 */ 0x00,0x00,0x00,0x00,	/* number of NetrCharDevQSetInfo [in]s  = 0 */
/*    196 */ 0xff,0xff,0xff,0xff,
/*    200 */ 0x00,0x00,0x00,0x01,	/* number of NetrCharDevQSetInfo [out]s = 1 */
/*    204 */ 0x00,0x00,0x04,0x58,	/* index  of NetrCharDevQSetInfo [out]s = 1112 */
/*    208 */ 0x00,0x00,0x00,0x00,	/* operation NetrCharDevQPurge flags */
/*    212 */ 0x00,0x00,0x00,0x02,	/* number of NetrCharDevQPurge params = 2 */
/*    216 */ 0x00,0x00,0x00,0x00,	/* number of NetrCharDevQPurge [in]s  = 0 */
/*    220 */ 0xff,0xff,0xff,0xff,
/*    224 */ 0x00,0x00,0x00,0x01,	/* number of NetrCharDevQPurge [out]s = 1 */
/*    228 */ 0x00,0x00,0x04,0x60,	/* index  of NetrCharDevQPurge [out]s = 1120 */
/*    232 */ 0x00,0x00,0x00,0x00,	/* operation NetrCharDevQPurgeSelf flags */
/*    236 */ 0x00,0x00,0x00,0x02,	/* number of NetrCharDevQPurgeSelf params = 2 */
/*    240 */ 0x00,0x00,0x00,0x00,	/* number of NetrCharDevQPurgeSelf [in]s  = 0 */
/*    244 */ 0xff,0xff,0xff,0xff,
/*    248 */ 0x00,0x00,0x00,0x01,	/* number of NetrCharDevQPurgeSelf [out]s = 1 */
/*    252 */ 0x00,0x00,0x04,0x68,	/* index  of NetrCharDevQPurgeSelf [out]s = 1128 */
/*    256 */ 0x00,0x00,0x00,0x00,	/* operation NetrConnectionEnum flags */
/*    260 */ 0x00,0x00,0x00,0x02,	/* number of NetrConnectionEnum params = 2 */
/*    264 */ 0x00,0x00,0x00,0x00,	/* number of NetrConnectionEnum [in]s  = 0 */
/*    268 */ 0xff,0xff,0xff,0xff,
/*    272 */ 0x00,0x00,0x00,0x01,	/* number of NetrConnectionEnum [out]s = 1 */
/*    276 */ 0x00,0x00,0x04,0x70,	/* index  of NetrConnectionEnum [out]s = 1136 */
/*    280 */ 0x00,0x00,0x00,0x00,	/* operation NetrFileEnum flags */
/*    284 */ 0x00,0x00,0x00,0x02,	/* number of NetrFileEnum params = 2 */
/*    288 */ 0x00,0x00,0x00,0x00,	/* number of NetrFileEnum [in]s  = 0 */
/*    292 */ 0xff,0xff,0xff,0xff,
/*    296 */ 0x00,0x00,0x00,0x01,	/* number of NetrFileEnum [out]s = 1 */
/*    300 */ 0x00,0x00,0x04,0x78,	/* index  of NetrFileEnum [out]s = 1144 */
/*    304 */ 0x00,0x00,0x00,0x00,	/* operation NetrFileGetInfo flags */
/*    308 */ 0x00,0x00,0x00,0x02,	/* number of NetrFileGetInfo params = 2 */
/*    312 */ 0x00,0x00,0x00,0x00,	/* number of NetrFileGetInfo [in]s  = 0 */
/*    316 */ 0xff,0xff,0xff,0xff,
/*    320 */ 0x00,0x00,0x00,0x01,	/* number of NetrFileGetInfo [out]s = 1 */
/*    324 */ 0x00,0x00,0x04,0x80,	/* index  of NetrFileGetInfo [out]s = 1152 */
/*    328 */ 0x00,0x00,0x00,0x00,	/* operation NetrFileClose flags */
/*    332 */ 0x00,0x00,0x00,0x02,	/* number of NetrFileClose params = 2 */
/*    336 */ 0x00,0x00,0x00,0x00,	/* number of NetrFileClose [in]s  = 0 */
/*    340 */ 0xff,0xff,0xff,0xff,
/*    344 */ 0x00,0x00,0x00,0x01,	/* number of NetrFileClose [out]s = 1 */
/*    348 */ 0x00,0x00,0x04,0x88,	/* index  of NetrFileClose [out]s = 1160 */
/*    352 */ 0x00,0x00,0x00,0x00,	/* operation NetrSessionEnum flags */
/*    356 */ 0x00,0x00,0x00,0x02,	/* number of NetrSessionEnum params = 2 */
/*    360 */ 0x00,0x00,0x00,0x00,	/* number of NetrSessionEnum [in]s  = 0 */
/*    364 */ 0xff,0xff,0xff,0xff,
/*    368 */ 0x00,0x00,0x00,0x01,	/* number of NetrSessionEnum [out]s = 1 */
/*    372 */ 0x00,0x00,0x04,0x90,	/* index  of NetrSessionEnum [out]s = 1168 */
/*    376 */ 0x00,0x00,0x00,0x00,	/* operation NetrSessionDel flags */
/*    380 */ 0x00,0x00,0x00,0x02,	/* number of NetrSessionDel params = 2 */
/*    384 */ 0x00,0x00,0x00,0x00,	/* number of NetrSessionDel [in]s  = 0 */
/*    388 */ 0xff,0xff,0xff,0xff,
/*    392 */ 0x00,0x00,0x00,0x01,	/* number of NetrSessionDel [out]s = 1 */
/*    396 */ 0x00,0x00,0x04,0x98,	/* index  of NetrSessionDel [out]s = 1176 */
/*    400 */ 0x00,0x00,0x00,0x00,	/* operation NetrShareAdd flags */
/*    404 */ 0x00,0x00,0x00,0x02,	/* number of NetrShareAdd params = 2 */
/*    408 */ 0x00,0x00,0x00,0x00,	/* number of NetrShareAdd [in]s  = 0 */
/*    412 */ 0xff,0xff,0xff,0xff,
/*    416 */ 0x00,0x00,0x00,0x01,	/* number of NetrShareAdd [out]s = 1 */
/*    420 */ 0x00,0x00,0x04,0xa0,	/* index  of NetrShareAdd [out]s = 1184 */
/*    424 */ 0x00,0x00,0x00,0x00,	/* operation NetrShareEnum flags */
/*    428 */ 0x00,0x00,0x00,0x08,	/* number of NetrShareEnum params = 8 */
/*    432 */ 0x00,0x00,0x00,0x05,	/* number of NetrShareEnum [in]s  = 5 */
/*    436 */ 0x00,0x00,0x04,0xa8,	/* index  of NetrShareEnum [in]s  = 1192 */
/*    440 */ 0x00,0x00,0x00,0x05,	/* number of NetrShareEnum [out]s = 5 */
/*    444 */ 0x00,0x00,0x04,0xe8,	/* index  of NetrShareEnum [out]s = 1256 */
/*    448 */ 0x00,0x00,0x00,0x00,	/* operation NetrShareGetInfo flags */
/*    452 */ 0x00,0x00,0x00,0x02,	/* number of NetrShareGetInfo params = 2 */
/*    456 */ 0x00,0x00,0x00,0x00,	/* number of NetrShareGetInfo [in]s  = 0 */
/*    460 */ 0xff,0xff,0xff,0xff,
/*    464 */ 0x00,0x00,0x00,0x01,	/* number of NetrShareGetInfo [out]s = 1 */
/*    468 */ 0x00,0x00,0x05,0x20,	/* index  of NetrShareGetInfo [out]s = 1312 */
/*    472 */ 0x00,0x00,0x00,0x00,	/* operation NetrShareSetInfo flags */
/*    476 */ 0x00,0x00,0x00,0x02,	/* number of NetrShareSetInfo params = 2 */
/*    480 */ 0x00,0x00,0x00,0x00,	/* number of NetrShareSetInfo [in]s  = 0 */
/*    484 */ 0xff,0xff,0xff,0xff,
/*    488 */ 0x00,0x00,0x00,0x01,	/* number of NetrShareSetInfo [out]s = 1 */
/*    492 */ 0x00,0x00,0x05,0x28,	/* index  of NetrShareSetInfo [out]s = 1320 */
/*    496 */ 0x00,0x00,0x00,0x00,	/* operation NetrShareDel flags */
/*    500 */ 0x00,0x00,0x00,0x02,	/* number of NetrShareDel params = 2 */
/*    504 */ 0x00,0x00,0x00,0x00,	/* number of NetrShareDel [in]s  = 0 */
/*    508 */ 0xff,0xff,0xff,0xff,
/*    512 */ 0x00,0x00,0x00,0x01,	/* number of NetrShareDel [out]s = 1 */
/*    516 */ 0x00,0x00,0x05,0x30,	/* index  of NetrShareDel [out]s = 1328 */
/*    520 */ 0x00,0x00,0x00,0x00,	/* operation NetrShareDelSticky flags */
/*    524 */ 0x00,0x00,0x00,0x02,	/* number of NetrShareDelSticky params = 2 */
/*    528 */ 0x00,0x00,0x00,0x00,	/* number of NetrShareDelSticky [in]s  = 0 */
/*    532 */ 0xff,0xff,0xff,0xff,
/*    536 */ 0x00,0x00,0x00,0x01,	/* number of NetrShareDelSticky [out]s = 1 */
/*    540 */ 0x00,0x00,0x05,0x38,	/* index  of NetrShareDelSticky [out]s = 1336 */
/*    544 */ 0x00,0x00,0x00,0x00,	/* operation NetrShareCheck flags */
/*    548 */ 0x00,0x00,0x00,0x02,	/* number of NetrShareCheck params = 2 */
/*    552 */ 0x00,0x00,0x00,0x00,	/* number of NetrShareCheck [in]s  = 0 */
/*    556 */ 0xff,0xff,0xff,0xff,
/*    560 */ 0x00,0x00,0x00,0x01,	/* number of NetrShareCheck [out]s = 1 */
/*    564 */ 0x00,0x00,0x05,0x40,	/* index  of NetrShareCheck [out]s = 1344 */
/*    568 */ 0x00,0x00,0x00,0x00,	/* operation NetrServerGetInfo flags */
/*    572 */ 0x00,0x00,0x00,0x02,	/* number of NetrServerGetInfo params = 2 */
/*    576 */ 0x00,0x00,0x00,0x00,	/* number of NetrServerGetInfo [in]s  = 0 */
/*    580 */ 0xff,0xff,0xff,0xff,
/*    584 */ 0x00,0x00,0x00,0x01,	/* number of NetrServerGetInfo [out]s = 1 */
/*    588 */ 0x00,0x00,0x05,0x48,	/* index  of NetrServerGetInfo [out]s = 1352 */
/*    592 */ 0x00,0x00,0x00,0x00,	/* operation NetrServerSetInfo flags */
/*    596 */ 0x00,0x00,0x00,0x02,	/* number of NetrServerSetInfo params = 2 */
/*    600 */ 0x00,0x00,0x00,0x00,	/* number of NetrServerSetInfo [in]s  = 0 */
/*    604 */ 0xff,0xff,0xff,0xff,
/*    608 */ 0x00,0x00,0x00,0x01,	/* number of NetrServerSetInfo [out]s = 1 */
/*    612 */ 0x00,0x00,0x05,0x50,	/* index  of NetrServerSetInfo [out]s = 1360 */
/*    616 */ 0x00,0x00,0x00,0x00,	/* operation NetrServerDiskEnum flags */
/*    620 */ 0x00,0x00,0x00,0x02,	/* number of NetrServerDiskEnum params = 2 */
/*    624 */ 0x00,0x00,0x00,0x00,	/* number of NetrServerDiskEnum [in]s  = 0 */
/*    628 */ 0xff,0xff,0xff,0xff,
/*    632 */ 0x00,0x00,0x00,0x01,	/* number of NetrServerDiskEnum [out]s = 1 */
/*    636 */ 0x00,0x00,0x05,0x58,	/* index  of NetrServerDiskEnum [out]s = 1368 */
/*    640 */ 0x00,0x00,0x00,0x00,	/* operation NetrServerStatisticsGet flags */
/*    644 */ 0x00,0x00,0x00,0x02,	/* number of NetrServerStatisticsGet params = 2 */
/*    648 */ 0x00,0x00,0x00,0x00,	/* number of NetrServerStatisticsGet [in]s  = 0 */
/*    652 */ 0xff,0xff,0xff,0xff,
/*    656 */ 0x00,0x00,0x00,0x01,	/* number of NetrServerStatisticsGet [out]s = 1 */
/*    660 */ 0x00,0x00,0x05,0x60,	/* index  of NetrServerStatisticsGet [out]s = 1376 */
/*    664 */ 0x00,0x00,0x00,0x00,	/* operation NetrServerTransportAdd flags */
/*    668 */ 0x00,0x00,0x00,0x02,	/* number of NetrServerTransportAdd params = 2 */
/*    672 */ 0x00,0x00,0x00,0x00,	/* number of NetrServerTransportAdd [in]s  = 0 */
/*    676 */ 0xff,0xff,0xff,0xff,
/*    680 */ 0x00,0x00,0x00,0x01,	/* number of NetrServerTransportAdd [out]s = 1 */
/*    684 */ 0x00,0x00,0x05,0x68,	/* index  of NetrServerTransportAdd [out]s = 1384 */
/*    688 */ 0x00,0x00,0x00,0x00,	/* operation NetrServerTransportEnum flags */
/*    692 */ 0x00,0x00,0x00,0x02,	/* number of NetrServerTransportEnum params = 2 */
/*    696 */ 0x00,0x00,0x00,0x00,	/* number of NetrServerTransportEnum [in]s  = 0 */
/*    700 */ 0xff,0xff,0xff,0xff,
/*    704 */ 0x00,0x00,0x00,0x01,	/* number of NetrServerTransportEnum [out]s = 1 */
/*    708 */ 0x00,0x00,0x05,0x70,	/* index  of NetrServerTransportEnum [out]s = 1392 */
/*    712 */ 0x00,0x00,0x00,0x00,	/* operation NetrServerTransportDel flags */
/*    716 */ 0x00,0x00,0x00,0x02,	/* number of NetrServerTransportDel params = 2 */
/*    720 */ 0x00,0x00,0x00,0x00,	/* number of NetrServerTransportDel [in]s  = 0 */
/*    724 */ 0xff,0xff,0xff,0xff,
/*    728 */ 0x00,0x00,0x00,0x01,	/* number of NetrServerTransportDel [out]s = 1 */
/*    732 */ 0x00,0x00,0x05,0x78,	/* index  of NetrServerTransportDel [out]s = 1400 */
/*    736 */ 0x00,0x00,0x00,0x00,	/* operation NetrSetServerServiceBits flags */
/*    740 */ 0x00,0x00,0x00,0x02,	/* number of NetrSetServerServiceBits params = 2 */
/*    744 */ 0x00,0x00,0x00,0x00,	/* number of NetrSetServerServiceBits [in]s  = 0 */
/*    748 */ 0xff,0xff,0xff,0xff,
/*    752 */ 0x00,0x00,0x00,0x01,	/* number of NetrSetServerServiceBits [out]s = 1 */
/*    756 */ 0x00,0x00,0x05,0x80,	/* index  of NetrSetServerServiceBits [out]s = 1408 */
/*    760 */ 0x00,0x00,0x00,0x00,	/* operation NetrPathType flags */
/*    764 */ 0x00,0x00,0x00,0x02,	/* number of NetrPathType params = 2 */
/*    768 */ 0x00,0x00,0x00,0x00,	/* number of NetrPathType [in]s  = 0 */
/*    772 */ 0xff,0xff,0xff,0xff,
/*    776 */ 0x00,0x00,0x00,0x01,	/* number of NetrPathType [out]s = 1 */
/*    780 */ 0x00,0x00,0x05,0x88,	/* index  of NetrPathType [out]s = 1416 */
/*    784 */ 0x00,0x00,0x00,0x00,	/* operation NetrPathCanonicalize flags */
/*    788 */ 0x00,0x00,0x00,0x02,	/* number of NetrPathCanonicalize params = 2 */
/*    792 */ 0x00,0x00,0x00,0x00,	/* number of NetrPathCanonicalize [in]s  = 0 */
/*    796 */ 0xff,0xff,0xff,0xff,
/*    800 */ 0x00,0x00,0x00,0x01,	/* number of NetrPathCanonicalize [out]s = 1 */
/*    804 */ 0x00,0x00,0x05,0x90,	/* index  of NetrPathCanonicalize [out]s = 1424 */
/*    808 */ 0x00,0x00,0x00,0x00,	/* operation NetrPathCompare flags */
/*    812 */ 0x00,0x00,0x00,0x02,	/* number of NetrPathCompare params = 2 */
/*    816 */ 0x00,0x00,0x00,0x00,	/* number of NetrPathCompare [in]s  = 0 */
/*    820 */ 0xff,0xff,0xff,0xff,
/*    824 */ 0x00,0x00,0x00,0x01,	/* number of NetrPathCompare [out]s = 1 */
/*    828 */ 0x00,0x00,0x05,0x98,	/* index  of NetrPathCompare [out]s = 1432 */
/*    832 */ 0x00,0x00,0x00,0x00,	/* operation NetrNameValidate flags */
/*    836 */ 0x00,0x00,0x00,0x02,	/* number of NetrNameValidate params = 2 */
/*    840 */ 0x00,0x00,0x00,0x00,	/* number of NetrNameValidate [in]s  = 0 */
/*    844 */ 0xff,0xff,0xff,0xff,
/*    848 */ 0x00,0x00,0x00,0x01,	/* number of NetrNameValidate [out]s = 1 */
/*    852 */ 0x00,0x00,0x05,0xa0,	/* index  of NetrNameValidate [out]s = 1440 */
/*    856 */ 0x00,0x00,0x00,0x00,	/* operation NetrNameCanonicalize flags */
/*    860 */ 0x00,0x00,0x00,0x02,	/* number of NetrNameCanonicalize params = 2 */
/*    864 */ 0x00,0x00,0x00,0x00,	/* number of NetrNameCanonicalize [in]s  = 0 */
/*    868 */ 0xff,0xff,0xff,0xff,
/*    872 */ 0x00,0x00,0x00,0x01,	/* number of NetrNameCanonicalize [out]s = 1 */
/*    876 */ 0x00,0x00,0x05,0xa8,	/* index  of NetrNameCanonicalize [out]s = 1448 */
/*    880 */ 0x00,0x00,0x00,0x00,	/* operation NetrNameCompare flags */
/*    884 */ 0x00,0x00,0x00,0x02,	/* number of NetrNameCompare params = 2 */
/*    888 */ 0x00,0x00,0x00,0x00,	/* number of NetrNameCompare [in]s  = 0 */
/*    892 */ 0xff,0xff,0xff,0xff,
/*    896 */ 0x00,0x00,0x00,0x01,	/* number of NetrNameCompare [out]s = 1 */
/*    900 */ 0x00,0x00,0x05,0xb0,	/* index  of NetrNameCompare [out]s = 1456 */
/*    904 */ 0x00,0x00,0x00,0x00,	/* operation NetrShareEnumSticky flags */
/*    908 */ 0x00,0x00,0x00,0x08,	/* number of NetrShareEnumSticky params = 8 */
/*    912 */ 0x00,0x00,0x00,0x05,	/* number of NetrShareEnumSticky [in]s  = 5 */
/*    916 */ 0x00,0x00,0x05,0xb8,	/* index  of NetrShareEnumSticky [in]s  = 1464 */
/*    920 */ 0x00,0x00,0x00,0x04,	/* number of NetrShareEnumSticky [out]s = 4 */
/*    924 */ 0x00,0x00,0x05,0xf8,	/* index  of NetrShareEnumSticky [out]s = 1528 */
/*    928 */ 0x00,0x00,0x00,0x00,	/* operation NetrShareDelStart flags */
/*    932 */ 0x00,0x00,0x00,0x02,	/* number of NetrShareDelStart params = 2 */
/*    936 */ 0x00,0x00,0x00,0x00,	/* number of NetrShareDelStart [in]s  = 0 */
/*    940 */ 0xff,0xff,0xff,0xff,
/*    944 */ 0x00,0x00,0x00,0x01,	/* number of NetrShareDelStart [out]s = 1 */
/*    948 */ 0x00,0x00,0x06,0x28,	/* index  of NetrShareDelStart [out]s = 1576 */
/*    952 */ 0x00,0x00,0x00,0x00,	/* operation NetrShareDelCommit flags */
/*    956 */ 0x00,0x00,0x00,0x02,	/* number of NetrShareDelCommit params = 2 */
/*    960 */ 0x00,0x00,0x00,0x00,	/* number of NetrShareDelCommit [in]s  = 0 */
/*    964 */ 0xff,0xff,0xff,0xff,
/*    968 */ 0x00,0x00,0x00,0x01,	/* number of NetrShareDelCommit [out]s = 1 */
/*    972 */ 0x00,0x00,0x06,0x30,	/* index  of NetrShareDelCommit [out]s = 1584 */
/*    976 */ 0x00,0x00,0x00,0x00,	/* operation NetrGetFileSecurity flags */
/*    980 */ 0x00,0x00,0x00,0x02,	/* number of NetrGetFileSecurity params = 2 */
/*    984 */ 0x00,0x00,0x00,0x00,	/* number of NetrGetFileSecurity [in]s  = 0 */
/*    988 */ 0xff,0xff,0xff,0xff,
/*    992 */ 0x00,0x00,0x00,0x01,	/* number of NetrGetFileSecurity [out]s = 1 */
/*    996 */ 0x00,0x00,0x06,0x38,	/* index  of NetrGetFileSecurity [out]s = 1592 */
/*   1000 */ 0x00,0x00,0x00,0x00,	/* operation NetrSetFileSecurity flags */
/*   1004 */ 0x00,0x00,0x00,0x02,	/* number of NetrSetFileSecurity params = 2 */
/*   1008 */ 0x00,0x00,0x00,0x00,	/* number of NetrSetFileSecurity [in]s  = 0 */
/*   1012 */ 0xff,0xff,0xff,0xff,
/*   1016 */ 0x00,0x00,0x00,0x01,	/* number of NetrSetFileSecurity [out]s = 1 */
/*   1020 */ 0x00,0x00,0x06,0x40,	/* index  of NetrSetFileSecurity [out]s = 1600 */
/*   1024 */ 0x00,0x00,0x00,0x00,	/* operation NetrServerTransportAddEx flags */
/*   1028 */ 0x00,0x00,0x00,0x02,	/* number of NetrServerTransportAddEx params = 2 */
/*   1032 */ 0x00,0x00,0x00,0x00,	/* number of NetrServerTransportAddEx [in]s  = 0 */
/*   1036 */ 0xff,0xff,0xff,0xff,
/*   1040 */ 0x00,0x00,0x00,0x01,	/* number of NetrServerTransportAddEx [out]s = 1 */
/*   1044 */ 0x00,0x00,0x06,0x48,	/* index  of NetrServerTransportAddEx [out]s = 1608 */
/*   1048 */ 0x00,0x00,0x00,0x00,	/* operation NetrServerSetServiceBitsEx flags */
/*   1052 */ 0x00,0x00,0x00,0x02,	/* number of NetrServerSetServiceBitsEx params = 2 */
/*   1056 */ 0x00,0x00,0x00,0x00,	/* number of NetrServerSetServiceBitsEx [in]s  = 0 */
/*   1060 */ 0xff,0xff,0xff,0xff,
/*   1064 */ 0x00,0x00,0x00,0x01,	/* number of NetrServerSetServiceBitsEx [out]s = 1 */
/*   1068 */ 0x00,0x00,0x06,0x50,	/* index  of NetrServerSetServiceBitsEx [out]s = 1616 */
		/* Operation NetrCharDevEnum parameter NetrCharDevEnum */
/*   1072 */ 0x00,0x00,0x00,0x00,	/* long 0 index of parameter NetrCharDevEnum */
/*   1076 */ IDL_DT_ULONG,	/* unsigned32 NetrCharDevEnum */
/*   1077 */ IDL_DT_EOL,
		/* Operation NetrCharDevGetInfo parameter NetrCharDevGetInfo */
/*   1078 */ 0xff,0xff,	/* filler */
/*   1080 */ 0x00,0x00,0x00,0x00,	/* long 0 index of parameter NetrCharDevGetInfo */
/*   1084 */ IDL_DT_ULONG,	/* unsigned32 NetrCharDevGetInfo */
/*   1085 */ IDL_DT_EOL,
		/* Operation NetrCharDevControl parameter NetrCharDevControl */
/*   1086 */ 0xff,0xff,	/* filler */
/*   1088 */ 0x00,0x00,0x00,0x00,	/* long 0 index of parameter NetrCharDevControl */
/*   1092 */ IDL_DT_ULONG,	/* unsigned32 NetrCharDevControl */
/*   1093 */ IDL_DT_EOL,
		/* Operation NetrCharDevQEnum parameter NetrCharDevQEnum */
/*   1094 */ 0xff,0xff,	/* filler */
/*   1096 */ 0x00,0x00,0x00,0x00,	/* long 0 index of parameter NetrCharDevQEnum */
/*   1100 */ IDL_DT_ULONG,	/* unsigned32 NetrCharDevQEnum */
/*   1101 */ IDL_DT_EOL,
		/* Operation NetrCharDevQGetInfo parameter NetrCharDevQGetInfo */
/*   1102 */ 0xff,0xff,	/* filler */
/*   1104 */ 0x00,0x00,0x00,0x00,	/* long 0 index of parameter NetrCharDevQGetInfo */
/*   1108 */ IDL_DT_ULONG,	/* unsigned32 NetrCharDevQGetInfo */
/*   1109 */ IDL_DT_EOL,
		/* Operation NetrCharDevQSetInfo parameter NetrCharDevQSetInfo */
/*   1110 */ 0xff,0xff,	/* filler */
/*   1112 */ 0x00,0x00,0x00,0x00,	/* long 0 index of parameter NetrCharDevQSetInfo */
/*   1116 */ IDL_DT_ULONG,	/* unsigned32 NetrCharDevQSetInfo */
/*   1117 */ IDL_DT_EOL,
		/* Operation NetrCharDevQPurge parameter NetrCharDevQPurge */
/*   1118 */ 0xff,0xff,	/* filler */
/*   1120 */ 0x00,0x00,0x00,0x00,	/* long 0 index of parameter NetrCharDevQPurge */
/*   1124 */ IDL_DT_ULONG,	/* unsigned32 NetrCharDevQPurge */
/*   1125 */ IDL_DT_EOL,
		/* Operation NetrCharDevQPurgeSelf parameter NetrCharDevQPurgeSelf */
/*   1126 */ 0xff,0xff,	/* filler */
/*   1128 */ 0x00,0x00,0x00,0x00,	/* long 0 index of parameter NetrCharDevQPurgeSelf */
/*   1132 */ IDL_DT_ULONG,	/* unsigned32 NetrCharDevQPurgeSelf */
/*   1133 */ IDL_DT_EOL,
		/* Operation NetrConnectionEnum parameter NetrConnectionEnum */
/*   1134 */ 0xff,0xff,	/* filler */
/*   1136 */ 0x00,0x00,0x00,0x00,	/* long 0 index of parameter NetrConnectionEnum */
/*   1140 */ IDL_DT_ULONG,	/* unsigned32 NetrConnectionEnum */
/*   1141 */ IDL_DT_EOL,
		/* Operation NetrFileEnum parameter NetrFileEnum */
/*   1142 */ 0xff,0xff,	/* filler */
/*   1144 */ 0x00,0x00,0x00,0x00,	/* long 0 index of parameter NetrFileEnum */
/*   1148 */ IDL_DT_ULONG,	/* unsigned32 NetrFileEnum */
/*   1149 */ IDL_DT_EOL,
		/* Operation NetrFileGetInfo parameter NetrFileGetInfo */
/*   1150 */ 0xff,0xff,	/* filler */
/*   1152 */ 0x00,0x00,0x00,0x00,	/* long 0 index of parameter NetrFileGetInfo */
/*   1156 */ IDL_DT_ULONG,	/* unsigned32 NetrFileGetInfo */
/*   1157 */ IDL_DT_EOL,
		/* Operation NetrFileClose parameter NetrFileClose */
/*   1158 */ 0xff,0xff,	/* filler */
/*   1160 */ 0x00,0x00,0x00,0x00,	/* long 0 index of parameter NetrFileClose */
/*   1164 */ IDL_DT_ULONG,	/* unsigned32 NetrFileClose */
/*   1165 */ IDL_DT_EOL,
		/* Operation NetrSessionEnum parameter NetrSessionEnum */
/*   1166 */ 0xff,0xff,	/* filler */
/*   1168 */ 0x00,0x00,0x00,0x00,	/* long 0 index of parameter NetrSessionEnum */
/*   1172 */ IDL_DT_ULONG,	/* unsigned32 NetrSessionEnum */
/*   1173 */ IDL_DT_EOL,
		/* Operation NetrSessionDel parameter NetrSessionDel */
/*   1174 */ 0xff,0xff,	/* filler */
/*   1176 */ 0x00,0x00,0x00,0x00,	/* long 0 index of parameter NetrSessionDel */
/*   1180 */ IDL_DT_ULONG,	/* unsigned32 NetrSessionDel */
/*   1181 */ IDL_DT_EOL,
		/* Operation NetrShareAdd parameter NetrShareAdd */
/*   1182 */ 0xff,0xff,	/* filler */
/*   1184 */ 0x00,0x00,0x00,0x00,	/* long 0 index of parameter NetrShareAdd */
/*   1188 */ IDL_DT_ULONG,	/* unsigned32 NetrShareAdd */
/*   1189 */ IDL_DT_EOL,
		/* Operation NetrShareEnum parameter ServerName */
/*   1190 */ 0xff,0xff,	/* filler */
/*   1192 */ 0x00,0x00,0x00,0x02,	/* long 2 index of parameter ServerName */
/*   1196 */ IDL_DT_UNIQUE_PTR,
/*   1197 */ 0|IDL_PROP_DEP_INT,	/* properties */
/*   1198 */ 0xff,0xff,	/* filler */
/*   1200 */ 0x00,0x00,0x06,0x56,	/* long 1622 pointee index */
/*   1204 */ IDL_DT_EOL,
		/* Operation NetrShareEnum parameter Level */
/*   1205 */ 0xff,0xff,0xff,	/* filler */
/*   1208 */ 0x00,0x00,0x00,0x03,	/* long 3 index of parameter Level */
/*   1212 */ IDL_DT_PASSED_BY_REF,
/*   1213 */ IDL_DT_ULONG,	/* unsigned32 Level */
/*   1214 */ IDL_DT_EOL,
		/* Operation NetrShareEnum parameter ShareInfo */
/*   1215 */ 0xff,	/* filler */
/*   1216 */ 0x00,0x00,0x00,0x04,	/* long 4 index of parameter ShareInfo */
/*   1220 */ IDL_DT_PASSED_BY_REF,
/*   1221 */ IDL_DT_FIXED_STRUCT,
/*   1222 */ 0|IDL_PROP_DEP_INT|IDL_PROP_HAS_PTRS,	/* properties */
/*   1223 */ 0xff,	/* filler */
/*   1224 */ 0x00,0x00,0x06,0x64,	/* long 1636 type SHARE_ENUM_STRUCT index */
/*   1228 */ IDL_DT_EOL,
		/* Operation NetrShareEnum parameter PreferredMaximumLength */
/*   1229 */ 0xff,0xff,0xff,	/* filler */
/*   1232 */ 0x00,0x00,0x00,0x05,	/* long 5 index of parameter PreferredMaximumLength */
/*   1236 */ IDL_DT_ULONG,	/* unsigned32 PreferredMaximumLength */
/*   1237 */ IDL_DT_EOL,
		/* Operation NetrShareEnum parameter ResumeHandle */
/*   1238 */ 0xff,0xff,	/* filler */
/*   1240 */ 0x00,0x00,0x00,0x07,	/* long 7 index of parameter ResumeHandle */
/*   1244 */ IDL_DT_UNIQUE_PTR,
/*   1245 */ 0|IDL_PROP_DEP_INT,	/* properties */
/*   1246 */ 0xff,0xff,	/* filler */
/*   1248 */ 0x00,0x00,0x06,0x71,	/* long 1649 pointee index */
/*   1252 */ IDL_DT_EOL,
		/* Operation NetrShareEnum parameter Level */
/*   1253 */ 0xff,0xff,0xff,	/* filler */
/*   1256 */ 0x00,0x00,0x00,0x03,	/* long 3 index of parameter Level */
/*   1260 */ IDL_DT_PASSED_BY_REF,
/*   1261 */ IDL_DT_ULONG,	/* unsigned32 Level */
/*   1262 */ IDL_DT_EOL,
		/* Operation NetrShareEnum parameter ShareInfo */
/*   1263 */ 0xff,	/* filler */
/*   1264 */ 0x00,0x00,0x00,0x04,	/* long 4 index of parameter ShareInfo */
/*   1268 */ IDL_DT_PASSED_BY_REF,
/*   1269 */ IDL_DT_FIXED_STRUCT,
/*   1270 */ 0|IDL_PROP_DEP_INT|IDL_PROP_HAS_PTRS,	/* properties */
/*   1271 */ 0xff,	/* filler */
/*   1272 */ 0x00,0x00,0x06,0x64,	/* long 1636 type SHARE_ENUM_STRUCT index */
/*   1276 */ IDL_DT_EOL,
		/* Operation NetrShareEnum parameter TotalEntries */
/*   1277 */ 0xff,0xff,0xff,	/* filler */
/*   1280 */ 0x00,0x00,0x00,0x06,	/* long 6 index of parameter TotalEntries */
/*   1284 */ IDL_DT_PASSED_BY_REF,
/*   1285 */ IDL_DT_ULONG,	/* unsigned32 TotalEntries */
/*   1286 */ IDL_DT_EOL,
		/* Operation NetrShareEnum parameter ResumeHandle */
/*   1287 */ 0xff,	/* filler */
/*   1288 */ 0x00,0x00,0x00,0x07,	/* long 7 index of parameter ResumeHandle */
/*   1292 */ IDL_DT_UNIQUE_PTR,
/*   1293 */ 0|IDL_PROP_DEP_INT,	/* properties */
/*   1294 */ 0xff,0xff,	/* filler */
/*   1296 */ 0x00,0x00,0x06,0x72,	/* long 1650 pointee index */
/*   1300 */ IDL_DT_EOL,
		/* Operation NetrShareEnum parameter NetrShareEnum */
/*   1301 */ 0xff,0xff,0xff,	/* filler */
/*   1304 */ 0x00,0x00,0x00,0x00,	/* long 0 index of parameter NetrShareEnum */
/*   1308 */ IDL_DT_ULONG,	/* unsigned32 NetrShareEnum */
/*   1309 */ IDL_DT_EOL,
		/* Operation NetrShareGetInfo parameter NetrShareGetInfo */
/*   1310 */ 0xff,0xff,	/* filler */
/*   1312 */ 0x00,0x00,0x00,0x00,	/* long 0 index of parameter NetrShareGetInfo */
/*   1316 */ IDL_DT_ULONG,	/* unsigned32 NetrShareGetInfo */
/*   1317 */ IDL_DT_EOL,
		/* Operation NetrShareSetInfo parameter NetrShareSetInfo */
/*   1318 */ 0xff,0xff,	/* filler */
/*   1320 */ 0x00,0x00,0x00,0x00,	/* long 0 index of parameter NetrShareSetInfo */
/*   1324 */ IDL_DT_ULONG,	/* unsigned32 NetrShareSetInfo */
/*   1325 */ IDL_DT_EOL,
		/* Operation NetrShareDel parameter NetrShareDel */
/*   1326 */ 0xff,0xff,	/* filler */
/*   1328 */ 0x00,0x00,0x00,0x00,	/* long 0 index of parameter NetrShareDel */
/*   1332 */ IDL_DT_ULONG,	/* unsigned32 NetrShareDel */
/*   1333 */ IDL_DT_EOL,
		/* Operation NetrShareDelSticky parameter NetrShareDelSticky */
/*   1334 */ 0xff,0xff,	/* filler */
/*   1336 */ 0x00,0x00,0x00,0x00,	/* long 0 index of parameter NetrShareDelSticky */
/*   1340 */ IDL_DT_ULONG,	/* unsigned32 NetrShareDelSticky */
/*   1341 */ IDL_DT_EOL,
		/* Operation NetrShareCheck parameter NetrShareCheck */
/*   1342 */ 0xff,0xff,	/* filler */
/*   1344 */ 0x00,0x00,0x00,0x00,	/* long 0 index of parameter NetrShareCheck */
/*   1348 */ IDL_DT_ULONG,	/* unsigned32 NetrShareCheck */
/*   1349 */ IDL_DT_EOL,
		/* Operation NetrServerGetInfo parameter NetrServerGetInfo */
/*   1350 */ 0xff,0xff,	/* filler */
/*   1352 */ 0x00,0x00,0x00,0x00,	/* long 0 index of parameter NetrServerGetInfo */
/*   1356 */ IDL_DT_ULONG,	/* unsigned32 NetrServerGetInfo */
/*   1357 */ IDL_DT_EOL,
		/* Operation NetrServerSetInfo parameter NetrServerSetInfo */
/*   1358 */ 0xff,0xff,	/* filler */
/*   1360 */ 0x00,0x00,0x00,0x00,	/* long 0 index of parameter NetrServerSetInfo */
/*   1364 */ IDL_DT_ULONG,	/* unsigned32 NetrServerSetInfo */
/*   1365 */ IDL_DT_EOL,
		/* Operation NetrServerDiskEnum parameter NetrServerDiskEnum */
/*   1366 */ 0xff,0xff,	/* filler */
/*   1368 */ 0x00,0x00,0x00,0x00,	/* long 0 index of parameter NetrServerDiskEnum */
/*   1372 */ IDL_DT_ULONG,	/* unsigned32 NetrServerDiskEnum */
/*   1373 */ IDL_DT_EOL,
		/* Operation NetrServerStatisticsGet parameter NetrServerStatisticsGet */
/*   1374 */ 0xff,0xff,	/* filler */
/*   1376 */ 0x00,0x00,0x00,0x00,	/* long 0 index of parameter NetrServerStatisticsGet */
/*   1380 */ IDL_DT_ULONG,	/* unsigned32 NetrServerStatisticsGet */
/*   1381 */ IDL_DT_EOL,
		/* Operation NetrServerTransportAdd parameter NetrServerTransportAdd */
/*   1382 */ 0xff,0xff,	/* filler */
/*   1384 */ 0x00,0x00,0x00,0x00,	/* long 0 index of parameter NetrServerTransportAdd */
/*   1388 */ IDL_DT_ULONG,	/* unsigned32 NetrServerTransportAdd */
/*   1389 */ IDL_DT_EOL,
		/* Operation NetrServerTransportEnum parameter NetrServerTransportEnum */
/*   1390 */ 0xff,0xff,	/* filler */
/*   1392 */ 0x00,0x00,0x00,0x00,	/* long 0 index of parameter NetrServerTransportEnum */
/*   1396 */ IDL_DT_ULONG,	/* unsigned32 NetrServerTransportEnum */
/*   1397 */ IDL_DT_EOL,
		/* Operation NetrServerTransportDel parameter NetrServerTransportDel */
/*   1398 */ 0xff,0xff,	/* filler */
/*   1400 */ 0x00,0x00,0x00,0x00,	/* long 0 index of parameter NetrServerTransportDel */
/*   1404 */ IDL_DT_ULONG,	/* unsigned32 NetrServerTransportDel */
/*   1405 */ IDL_DT_EOL,
		/* Operation NetrSetServerServiceBits parameter NetrSetServerServiceBits */
/*   1406 */ 0xff,0xff,	/* filler */
/*   1408 */ 0x00,0x00,0x00,0x00,	/* long 0 index of parameter NetrSetServerServiceBits */
/*   1412 */ IDL_DT_ULONG,	/* unsigned32 NetrSetServerServiceBits */
/*   1413 */ IDL_DT_EOL,
		/* Operation NetrPathType parameter NetrPathType */
/*   1414 */ 0xff,0xff,	/* filler */
/*   1416 */ 0x00,0x00,0x00,0x00,	/* long 0 index of parameter NetrPathType */
/*   1420 */ IDL_DT_ULONG,	/* unsigned32 NetrPathType */
/*   1421 */ IDL_DT_EOL,
		/* Operation NetrPathCanonicalize parameter NetrPathCanonicalize */
/*   1422 */ 0xff,0xff,	/* filler */
/*   1424 */ 0x00,0x00,0x00,0x00,	/* long 0 index of parameter NetrPathCanonicalize */
/*   1428 */ IDL_DT_ULONG,	/* unsigned32 NetrPathCanonicalize */
/*   1429 */ IDL_DT_EOL,
		/* Operation NetrPathCompare parameter NetrPathCompare */
/*   1430 */ 0xff,0xff,	/* filler */
/*   1432 */ 0x00,0x00,0x00,0x00,	/* long 0 index of parameter NetrPathCompare */
/*   1436 */ IDL_DT_ULONG,	/* unsigned32 NetrPathCompare */
/*   1437 */ IDL_DT_EOL,
		/* Operation NetrNameValidate parameter NetrNameValidate */
/*   1438 */ 0xff,0xff,	/* filler */
/*   1440 */ 0x00,0x00,0x00,0x00,	/* long 0 index of parameter NetrNameValidate */
/*   1444 */ IDL_DT_ULONG,	/* unsigned32 NetrNameValidate */
/*   1445 */ IDL_DT_EOL,
		/* Operation NetrNameCanonicalize parameter NetrNameCanonicalize */
/*   1446 */ 0xff,0xff,	/* filler */
/*   1448 */ 0x00,0x00,0x00,0x00,	/* long 0 index of parameter NetrNameCanonicalize */
/*   1452 */ IDL_DT_ULONG,	/* unsigned32 NetrNameCanonicalize */
/*   1453 */ IDL_DT_EOL,
		/* Operation NetrNameCompare parameter NetrNameCompare */
/*   1454 */ 0xff,0xff,	/* filler */
/*   1456 */ 0x00,0x00,0x00,0x00,	/* long 0 index of parameter NetrNameCompare */
/*   1460 */ IDL_DT_ULONG,	/* unsigned32 NetrNameCompare */
/*   1461 */ IDL_DT_EOL,
		/* Operation NetrShareEnumSticky parameter ServerName */
/*   1462 */ 0xff,0xff,	/* filler */
/*   1464 */ 0x00,0x00,0x00,0x02,	/* long 2 index of parameter ServerName */
/*   1468 */ IDL_DT_UNIQUE_PTR,
/*   1469 */ 0|IDL_PROP_DEP_INT,	/* properties */
/*   1470 */ 0xff,0xff,	/* filler */
/*   1472 */ 0x00,0x00,0x06,0x73,	/* long 1651 pointee index */
/*   1476 */ IDL_DT_EOL,
		/* Operation NetrShareEnumSticky parameter Level */
/*   1477 */ 0xff,0xff,0xff,	/* filler */
/*   1480 */ 0x00,0x00,0x00,0x03,	/* long 3 index of parameter Level */
/*   1484 */ IDL_DT_ULONG,	/* unsigned32 Level */
/*   1485 */ IDL_DT_EOL,
		/* Operation NetrShareEnumSticky parameter ShareInfo */
/*   1486 */ 0xff,0xff,	/* filler */
/*   1488 */ 0x00,0x00,0x00,0x04,	/* long 4 index of parameter ShareInfo */
/*   1492 */ IDL_DT_PASSED_BY_REF,
/*   1493 */ IDL_DT_FIXED_STRUCT,
/*   1494 */ 0|IDL_PROP_DEP_INT|IDL_PROP_HAS_PTRS,	/* properties */
/*   1495 */ 0xff,	/* filler */
/*   1496 */ 0x00,0x00,0x06,0x64,	/* long 1636 type SHARE_ENUM_STRUCT index */
/*   1500 */ IDL_DT_EOL,
		/* Operation NetrShareEnumSticky parameter PreferredMaximumLength */
/*   1501 */ 0xff,0xff,0xff,	/* filler */
/*   1504 */ 0x00,0x00,0x00,0x05,	/* long 5 index of parameter PreferredMaximumLength */
/*   1508 */ IDL_DT_ULONG,	/* unsigned32 PreferredMaximumLength */
/*   1509 */ IDL_DT_EOL,
		/* Operation NetrShareEnumSticky parameter ResumeHandle */
/*   1510 */ 0xff,0xff,	/* filler */
/*   1512 */ 0x00,0x00,0x00,0x07,	/* long 7 index of parameter ResumeHandle */
/*   1516 */ IDL_DT_UNIQUE_PTR,
/*   1517 */ 0|IDL_PROP_DEP_INT,	/* properties */
/*   1518 */ 0xff,0xff,	/* filler */
/*   1520 */ 0x00,0x00,0x06,0x80,	/* long 1664 pointee index */
/*   1524 */ IDL_DT_EOL,
		/* Operation NetrShareEnumSticky parameter ShareInfo */
/*   1525 */ 0xff,0xff,0xff,	/* filler */
/*   1528 */ 0x00,0x00,0x00,0x04,	/* long 4 index of parameter ShareInfo */
/*   1532 */ IDL_DT_PASSED_BY_REF,
/*   1533 */ IDL_DT_FIXED_STRUCT,
/*   1534 */ 0|IDL_PROP_DEP_INT|IDL_PROP_HAS_PTRS,	/* properties */
/*   1535 */ 0xff,	/* filler */
/*   1536 */ 0x00,0x00,0x06,0x64,	/* long 1636 type SHARE_ENUM_STRUCT index */
/*   1540 */ IDL_DT_EOL,
		/* Operation NetrShareEnumSticky parameter TotalEntries */
/*   1541 */ 0xff,0xff,0xff,	/* filler */
/*   1544 */ 0x00,0x00,0x00,0x06,	/* long 6 index of parameter TotalEntries */
/*   1548 */ IDL_DT_PASSED_BY_REF,
/*   1549 */ IDL_DT_ULONG,	/* unsigned32 TotalEntries */
/*   1550 */ IDL_DT_EOL,
		/* Operation NetrShareEnumSticky parameter ResumeHandle */
/*   1551 */ 0xff,	/* filler */
/*   1552 */ 0x00,0x00,0x00,0x07,	/* long 7 index of parameter ResumeHandle */
/*   1556 */ IDL_DT_UNIQUE_PTR,
/*   1557 */ 0|IDL_PROP_DEP_INT,	/* properties */
/*   1558 */ 0xff,0xff,	/* filler */
/*   1560 */ 0x00,0x00,0x06,0x81,	/* long 1665 pointee index */
/*   1564 */ IDL_DT_EOL,
		/* Operation NetrShareEnumSticky parameter NetrShareEnumSticky */
/*   1565 */ 0xff,0xff,0xff,	/* filler */
/*   1568 */ 0x00,0x00,0x00,0x00,	/* long 0 index of parameter NetrShareEnumSticky */
/*   1572 */ IDL_DT_ULONG,	/* unsigned32 NetrShareEnumSticky */
/*   1573 */ IDL_DT_EOL,
		/* Operation NetrShareDelStart parameter NetrShareDelStart */
/*   1574 */ 0xff,0xff,	/* filler */
/*   1576 */ 0x00,0x00,0x00,0x00,	/* long 0 index of parameter NetrShareDelStart */
/*   1580 */ IDL_DT_ULONG,	/* unsigned32 NetrShareDelStart */
/*   1581 */ IDL_DT_EOL,
		/* Operation NetrShareDelCommit parameter NetrShareDelCommit */
/*   1582 */ 0xff,0xff,	/* filler */
/*   1584 */ 0x00,0x00,0x00,0x00,	/* long 0 index of parameter NetrShareDelCommit */
/*   1588 */ IDL_DT_ULONG,	/* unsigned32 NetrShareDelCommit */
/*   1589 */ IDL_DT_EOL,
		/* Operation NetrGetFileSecurity parameter NetrGetFileSecurity */
/*   1590 */ 0xff,0xff,	/* filler */
/*   1592 */ 0x00,0x00,0x00,0x00,	/* long 0 index of parameter NetrGetFileSecurity */
/*   1596 */ IDL_DT_ULONG,	/* unsigned32 NetrGetFileSecurity */
/*   1597 */ IDL_DT_EOL,
		/* Operation NetrSetFileSecurity parameter NetrSetFileSecurity */
/*   1598 */ 0xff,0xff,	/* filler */
/*   1600 */ 0x00,0x00,0x00,0x00,	/* long 0 index of parameter NetrSetFileSecurity */
/*   1604 */ IDL_DT_ULONG,	/* unsigned32 NetrSetFileSecurity */
/*   1605 */ IDL_DT_EOL,
		/* Operation NetrServerTransportAddEx parameter NetrServerTransportAddEx */
/*   1606 */ 0xff,0xff,	/* filler */
/*   1608 */ 0x00,0x00,0x00,0x00,	/* long 0 index of parameter NetrServerTransportAddEx */
/*   1612 */ IDL_DT_ULONG,	/* unsigned32 NetrServerTransportAddEx */
/*   1613 */ IDL_DT_EOL,
		/* Operation NetrServerSetServiceBitsEx parameter NetrServerSetServiceBitsEx */
/*   1614 */ 0xff,0xff,	/* filler */
/*   1616 */ 0x00,0x00,0x00,0x00,	/* long 0 index of parameter NetrServerSetServiceBitsEx */
/*   1620 */ IDL_DT_ULONG,	/* unsigned32 NetrServerSetServiceBitsEx */
/*   1621 */ IDL_DT_EOL,
/*   1622 */ IDL_DT_STRING,
/*   1623 */ IDL_DT_OPEN_ARRAY,
/*   1624 */ 0|IDL_PROP_DEP_INT,	/* properties */
/*   1625 */ 0xff,0xff,0xff,	/* filler */
/*   1628 */ 0x00,0x00,0x07,0x77,	/* long 1911 full array index */
/*   1632 */ 0x00,0x00,0x07,0x77,	/* long 1911 flat array index */
		/* struct SHARE_ENUM_STRUCT definition */
/*   1636 */ 0x00,0x00,0x00,0x01,	/* long 1 offset vector index */
/*   1640 */ IDL_DT_NDR_ALIGN_4,
/*   1641 */ IDL_DT_ENC_UNION,
/*   1642 */ 0|IDL_PROP_DEP_INT|IDL_PROP_HAS_PTRS,	/* properties */
/*   1643 */ 0xff,	/* filler */
/*   1644 */ 0x00,0x00,0x06,0xa8,	/* long 1704 type SHARE_ENUM_UNION index */
/*   1648 */ IDL_DT_EOL,
/*   1649 */ IDL_DT_ULONG,	/* unsigned32 ResumeHandle */
/*   1650 */ IDL_DT_ULONG,	/* unsigned32 ResumeHandle */
/*   1651 */ IDL_DT_STRING,
/*   1652 */ IDL_DT_OPEN_ARRAY,
/*   1653 */ 0|IDL_PROP_DEP_INT,	/* properties */
/*   1654 */ 0xff,0xff,	/* filler */
/*   1656 */ 0x00,0x00,0x06,0x83,	/* long 1667 full array index */
/*   1660 */ 0x00,0x00,0x06,0x83,	/* long 1667 flat array index */
/*   1664 */ IDL_DT_ULONG,	/* unsigned32 ResumeHandle */
/*   1665 */ IDL_DT_ULONG,	/* unsigned32 ResumeHandle */
		/* array anon_array full instance */
/*   1666 */ 0xff,	/* filler */
/*   1667 */ 1,	/* num dimensions */
/*   1668 */ IDL_BOUND_FIXED,
/*   1669 */ 0xff,0xff,0xff,	/* filler */
/*   1672 */ 0x00,0x00,0x00,0x00,	/* long 0 fixed bound */
/*   1676 */ IDL_BOUND_STRING,
/*   1677 */ 2,	/* octetsize */
/*   1678 */ 0xff,0xff,	/* filler */
/*   1680 */ 0xff,0xff,0xff,0xff,	/* long -1 dummy value */
/*   1684 */ IDL_LIMIT_FIXED,
/*   1685 */ 0xff,0xff,0xff,	/* filler */
/*   1688 */ 0x00,0x00,0x00,0x00,	/* long 0 fixed limit */
/*   1692 */ IDL_LIMIT_STRING,
/*   1693 */ 2,	/* octetsize */
/*   1694 */ 0xff,0xff,	/* filler */
/*   1696 */ 0xff,0xff,0xff,0xff,	/* long -1 dummy value */
/*   1700 */ IDL_DT_USHORT,
		/* union SHARE_ENUM_UNION definition */
/*   1701 */ 0xff,0xff,0xff,	/* filler */
/*   1704 */ 0x00,0x00,0x00,0x03,	/* long 3 offset vector index */
/*   1708 */ IDL_DT_ULONG,	/* unsigned32 switch type */
/*   1709 */ 0xff,0xff,0xff,	/* filler */
/*   1712 */ 0x00,0x00,0x00,0x01,	/* long 1 number of arms */
/*   1716 */ 0x00,0x00,0x00,0x01,	/* long 1 switch value */
/*   1720 */ IDL_DT_FULL_PTR,
/*   1721 */ 0|IDL_PROP_DEP_INT|IDL_PROP_HAS_PTRS,	/* properties */
/*   1722 */ 0xff,0xff,	/* filler */
/*   1724 */ 0x00,0x00,0x06,0xc5,	/* long 1733 pointee index */
/*   1728 */ 0xff,0xff,0xff,0xff,	/* filler */
/*   1732 */ IDL_DT_DOES_NOT_EXIST,
/*   1733 */ IDL_DT_FIXED_STRUCT,
/*   1734 */ 0|IDL_PROP_DEP_INT|IDL_PROP_HAS_PTRS,	/* properties */
/*   1735 */ 0xff,	/* filler */
/*   1736 */ 0x00,0x00,0x06,0xcc,	/* long 1740 type SHARE_INFO_1_CONTAINER index */
		/* struct SHARE_INFO_1_CONTAINER definition */
/*   1740 */ 0x00,0x00,0x00,0x05,	/* long 5 offset vector index */
/*   1744 */ IDL_DT_NDR_ALIGN_4,
/*   1745 */ IDL_DT_ULONG,	/* unsigned32 share_count */
/*   1746 */ IDL_DT_UNIQUE_PTR,
/*   1747 */ 0|IDL_PROP_DEP_INT|IDL_PROP_HAS_PTRS,	/* properties */
/*   1748 */ 0x00,0x00,0x06,0xd9,	/* long 1753 pointee index */
/*   1752 */ IDL_DT_EOL,
/*   1753 */ IDL_DT_CONF_ARRAY,
/*   1754 */ 0|IDL_PROP_DEP_INT|IDL_PROP_HAS_PTRS,	/* properties */
/*   1755 */ 0xff,	/* filler */
/*   1756 */ 0x00,0x00,0x06,0xe7,	/* long 1767 full array index */
/*   1760 */ 0x00,0x00,0x06,0xe7,	/* long 1767 flat array index */
		/* array anon_array full instance */
/*   1764 */ 0xff,0xff,0xff,	/* filler */
/*   1767 */ 1,	/* num dimensions */
/*   1768 */ IDL_BOUND_FIXED,
/*   1769 */ 0xff,0xff,0xff,	/* filler */
/*   1772 */ 0x00,0x00,0x00,0x00,	/* long 0 fixed bound */
/*   1776 */ IDL_BOUND_SIZE_IS,
/*   1777 */ IDL_DT_ULONG,	/* unsigned32 bound type */
/*   1778 */ 0xff,0xff,	/* filler */
/*   1780 */ 0x00,0x00,0x00,0x01,	/* long 1 field number of size_is reference */
/*   1784 */ IDL_DT_FIXED_STRUCT,
/*   1785 */ 0|IDL_PROP_DEP_INT|IDL_PROP_HAS_PTRS,	/* properties */
/*   1786 */ 0xff,0xff,	/* filler */
/*   1788 */ 0x00,0x00,0x07,0x00,	/* long 1792 type SHARE_INFO_1 index */
		/* struct SHARE_INFO_1 definition */
/*   1792 */ 0x00,0x00,0x00,0x08,	/* long 8 offset vector index */
/*   1796 */ IDL_DT_NDR_ALIGN_4,
/*   1797 */ IDL_DT_REF_PTR,
/*   1798 */ 0|IDL_PROP_DEP_INT,	/* properties */
/*   1799 */ 0xff,	/* filler */
/*   1800 */ 0x00,0x00,0x07,0x45,	/* long 1861 pointee index */
/*   1804 */ IDL_DT_ULONG,	/* unsigned32 shi1_type */
/*   1805 */ IDL_DT_REF_PTR,
/*   1806 */ 0|IDL_PROP_DEP_INT,	/* properties */
/*   1807 */ 0xff,	/* filler */
/*   1808 */ 0x00,0x00,0x07,0x15,	/* long 1813 pointee index */
/*   1812 */ IDL_DT_EOL,
/*   1813 */ IDL_DT_STRING,
/*   1814 */ IDL_DT_OPEN_ARRAY,
/*   1815 */ 0|IDL_PROP_DEP_INT,	/* properties */
/*   1816 */ 0x00,0x00,0x07,0x23,	/* long 1827 full array index */
/*   1820 */ 0x00,0x00,0x07,0x23,	/* long 1827 flat array index */
		/* array anon_array full instance */
/*   1824 */ 0xff,0xff,0xff,	/* filler */
/*   1827 */ 1,	/* num dimensions */
/*   1828 */ IDL_BOUND_FIXED,
/*   1829 */ 0xff,0xff,0xff,	/* filler */
/*   1832 */ 0x00,0x00,0x00,0x00,	/* long 0 fixed bound */
/*   1836 */ IDL_BOUND_STRING,
/*   1837 */ 2,	/* octetsize */
/*   1838 */ 0xff,0xff,	/* filler */
/*   1840 */ 0xff,0xff,0xff,0xff,	/* long -1 dummy value */
/*   1844 */ IDL_LIMIT_FIXED,
/*   1845 */ 0xff,0xff,0xff,	/* filler */
/*   1848 */ 0x00,0x00,0x00,0x00,	/* long 0 fixed limit */
/*   1852 */ IDL_LIMIT_STRING,
/*   1853 */ 2,	/* octetsize */
/*   1854 */ 0xff,0xff,	/* filler */
/*   1856 */ 0xff,0xff,0xff,0xff,	/* long -1 dummy value */
/*   1860 */ IDL_DT_USHORT,
/*   1861 */ IDL_DT_STRING,
/*   1862 */ IDL_DT_OPEN_ARRAY,
/*   1863 */ 0|IDL_PROP_DEP_INT,	/* properties */
/*   1864 */ 0x00,0x00,0x07,0x53,	/* long 1875 full array index */
/*   1868 */ 0x00,0x00,0x07,0x53,	/* long 1875 flat array index */
		/* array anon_array full instance */
/*   1872 */ 0xff,0xff,0xff,	/* filler */
/*   1875 */ 1,	/* num dimensions */
/*   1876 */ IDL_BOUND_FIXED,
/*   1877 */ 0xff,0xff,0xff,	/* filler */
/*   1880 */ 0x00,0x00,0x00,0x00,	/* long 0 fixed bound */
/*   1884 */ IDL_BOUND_STRING,
/*   1885 */ 2,	/* octetsize */
/*   1886 */ 0xff,0xff,	/* filler */
/*   1888 */ 0xff,0xff,0xff,0xff,	/* long -1 dummy value */
/*   1892 */ IDL_LIMIT_FIXED,
/*   1893 */ 0xff,0xff,0xff,	/* filler */
/*   1896 */ 0x00,0x00,0x00,0x00,	/* long 0 fixed limit */
/*   1900 */ IDL_LIMIT_STRING,
/*   1901 */ 2,	/* octetsize */
/*   1902 */ 0xff,0xff,	/* filler */
/*   1904 */ 0xff,0xff,0xff,0xff,	/* long -1 dummy value */
/*   1908 */ IDL_DT_USHORT,
		/* array anon_array full instance */
/*   1909 */ 0xff,0xff,	/* filler */
/*   1911 */ 1,	/* num dimensions */
/*   1912 */ IDL_BOUND_FIXED,
/*   1913 */ 0xff,0xff,0xff,	/* filler */
/*   1916 */ 0x00,0x00,0x00,0x00,	/* long 0 fixed bound */
/*   1920 */ IDL_BOUND_STRING,
/*   1921 */ 2,	/* octetsize */
/*   1922 */ 0xff,0xff,	/* filler */
/*   1924 */ 0xff,0xff,0xff,0xff,	/* long -1 dummy value */
/*   1928 */ IDL_LIMIT_FIXED,
/*   1929 */ 0xff,0xff,0xff,	/* filler */
/*   1932 */ 0x00,0x00,0x00,0x00,	/* long 0 fixed limit */
/*   1936 */ IDL_LIMIT_STRING,
/*   1937 */ 2,	/* octetsize */
/*   1938 */ 0xff,0xff,	/* filler */
/*   1940 */ 0xff,0xff,0xff,0xff,	/* long -1 dummy value */
/*   1944 */ IDL_DT_USHORT,
/*   1945 */ 0xff,0xff,0xff,	/* filler */
/*   1948 */ 0x00,0x00,0x00,0x10,	/* bug flags */
0		/* sentinel */
};


unsigned32 NetrShareEnum
(
    /* [in] */ handle_t h,
    /* [in] */ idl_ushort_int *ServerName,
    /* [in, out] */ unsigned32 *Level,
    /* [in, out] */ SHARE_ENUM_STRUCT *ShareInfo,
    /* [in] */ unsigned32 PreferredMaximumLength,
    /* [out] */ unsigned32 *TotalEntries,
    /* [in, out] */ unsigned32 *ResumeHandle
)
{
rpc_transfer_syntax_t IDL_transfer_syntax;
rpc_iovector_elt_t IDL_outs;
volatile ndr_ulong_int IDL_fault_code=error_status_ok;
volatile ndr_ulong_int IDL_user_fault_id;
volatile RPC_SS_THREADS_CANCEL_STATE_T IDL_async_cancel_state;
IDL_ms_t IDL_ms;
idl_byte IDL_stack_packet[IDL_STACK_PACKET_SIZE];
rpc_void_p_t IDL_param_vec[8];
unsigned32 IDL_function_result;

RPC_SS_INIT_CLIENT
RPC_SS_THREADS_DISABLE_ASYNC(IDL_async_cancel_state);
rpc_ss_init_marsh_state(IDL_type_vec, &IDL_ms);
IDL_ms.IDL_stack_packet_status = IDL_stack_packet_unused_k;
IDL_ms.IDL_stack_packet_addr = IDL_stack_packet;
TRY
IDL_ms.IDL_call_h = 0;
IDL_ms.IDL_elt_p = NULL;
IDL_ms.IDL_offset_vec = IDL_offset_vec;
IDL_ms.IDL_rtn_vec = IDL_rtn_vec;
IDL_param_vec[0] = (rpc_void_p_t)&IDL_function_result;
IDL_param_vec[1] = (rpc_void_p_t)&h;
IDL_param_vec[2] = (rpc_void_p_t)&ServerName;
IDL_param_vec[3] = (rpc_void_p_t)Level;
IDL_param_vec[4] = (rpc_void_p_t)ShareInfo;
IDL_param_vec[5] = (rpc_void_p_t)&PreferredMaximumLength;
IDL_param_vec[6] = (rpc_void_p_t)TotalEntries;
IDL_param_vec[7] = (rpc_void_p_t)&ResumeHandle;
IDL_ms.IDL_param_vec = IDL_param_vec;
IDL_ms.IDL_side = IDL_client_side_k;
IDL_ms.IDL_language = IDL_lang_c_k;
rpc_ss_init_node_table(&IDL_ms.IDL_node_table,&IDL_ms.IDL_mem_handle);
rpc_ss_mts_client_estab_alloc(&IDL_ms);
rpc_call_start((rpc_binding_handle_t) h, 0,
 (rpc_if_handle_t)&IDL_ifspec,15,(rpc_call_handle_t*)&IDL_ms.IDL_call_h,&IDL_transfer_syntax,(unsigned32*)&IDL_ms.IDL_status);
if (IDL_ms.IDL_status != error_status_ok) goto IDL_closedown;
rpc_ss_ndr_marsh_interp(
    5,	/* number of [in] parameters */
    1192,	/* type vector index of first [in] parameter */
    IDL_param_vec, &IDL_ms);
IDL_ms.IDL_elt_p = &IDL_outs;
rpc_call_transceive((rpc_call_handle_t)IDL_ms.IDL_call_h,(rpc_iovector_p_t)&IDL_ms.IDL_iovec,
  IDL_ms.IDL_elt_p,&IDL_ms.IDL_drep,(unsigned32*)&IDL_ms.IDL_status);
if (IDL_ms.IDL_status != error_status_ok)
{
IDL_ms.IDL_elt_p = NULL;
goto IDL_closedown;
}
rpc_ss_ndr_unmar_interp(
    5,	/* number of [out] parameters */
    1256,	/* type vector index of first [out] parameter */
    IDL_param_vec, &IDL_ms);
IDL_closedown: ;
CATCH(rpc_x_ss_pipe_comm_error)
FINALLY
rpc_ss_ndr_clean_up(&IDL_ms);
rpc_ss_call_end_2(&IDL_ms.IDL_call_h,&IDL_fault_code,&IDL_user_fault_id,&IDL_ms.IDL_status);
if (IDL_ms.IDL_mem_handle.memory)
{
 rpc_ss_mem_free(&IDL_ms.IDL_mem_handle);
}
rpc_ss_report_error_2(IDL_fault_code,IDL_user_fault_id,IDL_ms.IDL_status,
 (RPC_SS_THREADS_CANCEL_STATE_T *)&IDL_async_cancel_state, NULL, NULL, NULL, &IDL_ms);
RPC_SS_THREADS_RESTORE_ASYNC(IDL_async_cancel_state);
ENDTRY
return IDL_function_result;
}

unsigned32 NetrShareEnumSticky
(
    /* [in] */ handle_t h,
    /* [in] */ idl_ushort_int *ServerName,
    /* [in] */ unsigned32 Level,
    /* [in, out] */ SHARE_ENUM_STRUCT *ShareInfo,
    /* [in] */ unsigned32 PreferredMaximumLength,
    /* [out] */ unsigned32 *TotalEntries,
    /* [in, out] */ unsigned32 *ResumeHandle
)
{
rpc_transfer_syntax_t IDL_transfer_syntax;
rpc_iovector_elt_t IDL_outs;
volatile ndr_ulong_int IDL_fault_code=error_status_ok;
volatile ndr_ulong_int IDL_user_fault_id;
volatile RPC_SS_THREADS_CANCEL_STATE_T IDL_async_cancel_state;
IDL_ms_t IDL_ms;
idl_byte IDL_stack_packet[IDL_STACK_PACKET_SIZE];
rpc_void_p_t IDL_param_vec[8];
unsigned32 IDL_function_result;

RPC_SS_INIT_CLIENT
RPC_SS_THREADS_DISABLE_ASYNC(IDL_async_cancel_state);
rpc_ss_init_marsh_state(IDL_type_vec, &IDL_ms);
IDL_ms.IDL_stack_packet_status = IDL_stack_packet_unused_k;
IDL_ms.IDL_stack_packet_addr = IDL_stack_packet;
TRY
IDL_ms.IDL_call_h = 0;
IDL_ms.IDL_elt_p = NULL;
IDL_ms.IDL_offset_vec = IDL_offset_vec;
IDL_ms.IDL_rtn_vec = IDL_rtn_vec;
IDL_param_vec[0] = (rpc_void_p_t)&IDL_function_result;
IDL_param_vec[1] = (rpc_void_p_t)&h;
IDL_param_vec[2] = (rpc_void_p_t)&ServerName;
IDL_param_vec[3] = (rpc_void_p_t)&Level;
IDL_param_vec[4] = (rpc_void_p_t)ShareInfo;
IDL_param_vec[5] = (rpc_void_p_t)&PreferredMaximumLength;
IDL_param_vec[6] = (rpc_void_p_t)TotalEntries;
IDL_param_vec[7] = (rpc_void_p_t)&ResumeHandle;
IDL_ms.IDL_param_vec = IDL_param_vec;
IDL_ms.IDL_side = IDL_client_side_k;
IDL_ms.IDL_language = IDL_lang_c_k;
rpc_ss_init_node_table(&IDL_ms.IDL_node_table,&IDL_ms.IDL_mem_handle);
rpc_ss_mts_client_estab_alloc(&IDL_ms);
rpc_call_start((rpc_binding_handle_t) h, 0,
 (rpc_if_handle_t)&IDL_ifspec,35,(rpc_call_handle_t*)&IDL_ms.IDL_call_h,&IDL_transfer_syntax,(unsigned32*)&IDL_ms.IDL_status);
if (IDL_ms.IDL_status != error_status_ok) goto IDL_closedown;
rpc_ss_ndr_marsh_interp(
    5,	/* number of [in] parameters */
    1464,	/* type vector index of first [in] parameter */
    IDL_param_vec, &IDL_ms);
IDL_ms.IDL_elt_p = &IDL_outs;
rpc_call_transceive((rpc_call_handle_t)IDL_ms.IDL_call_h,(rpc_iovector_p_t)&IDL_ms.IDL_iovec,
  IDL_ms.IDL_elt_p,&IDL_ms.IDL_drep,(unsigned32*)&IDL_ms.IDL_status);
if (IDL_ms.IDL_status != error_status_ok)
{
IDL_ms.IDL_elt_p = NULL;
goto IDL_closedown;
}
rpc_ss_ndr_unmar_interp(
    4,	/* number of [out] parameters */
    1528,	/* type vector index of first [out] parameter */
    IDL_param_vec, &IDL_ms);
IDL_closedown: ;
CATCH(rpc_x_ss_pipe_comm_error)
FINALLY
rpc_ss_ndr_clean_up(&IDL_ms);
rpc_ss_call_end_2(&IDL_ms.IDL_call_h,&IDL_fault_code,&IDL_user_fault_id,&IDL_ms.IDL_status);
if (IDL_ms.IDL_mem_handle.memory)
{
 rpc_ss_mem_free(&IDL_ms.IDL_mem_handle);
}
rpc_ss_report_error_2(IDL_fault_code,IDL_user_fault_id,IDL_ms.IDL_status,
 (RPC_SS_THREADS_CANCEL_STATE_T *)&IDL_async_cancel_state, NULL, NULL, NULL, &IDL_ms);
RPC_SS_THREADS_RESTORE_ASYNC(IDL_async_cancel_state);
ENDTRY
return IDL_function_result;
}
#ifdef VMS
#pragma standard
#endif
