/* Generated by IDL compiler version OSF DCE T1.1.0-03 */
#ifndef codesets_v1_0_included
#define codesets_v1_0_included
#ifndef IDLBASE_H
#include <dce/idlbase.h>
#endif
#include <dce/rpc.h>
#include <dce/idl_es.h>

#ifdef __cplusplus
    extern "C" {
#endif

#ifndef nbase_v0_0_included
#include <dce/nbase.h>
#endif
#define nsi_max_attribute_len (4000)
extern void rpc__codesets_to_nscodesets(
    /* [in] */ idl_es_handle_t h,
    /* [in] */ idl_long_int num,
    /* [in] */ idl_byte codesets[]
);
extern void rpc__nscodesets_to_codesets(
    /* [in] */ idl_es_handle_t h,
    /* [in, out] */ idl_long_int *num,
    /* [out] */ idl_byte codesets[]
);
typedef struct codesets_v1_0_epv_t {
void (*rpc__codesets_to_nscodesets)(
    /* [in] */ idl_es_handle_t h,
    /* [in] */ idl_long_int num,
    /* [in] */ idl_byte codesets[]
);
void (*rpc__nscodesets_to_codesets)(
    /* [in] */ idl_es_handle_t h,
    /* [in, out] */ idl_long_int *num,
    /* [out] */ idl_byte codesets[]
);
} codesets_v1_0_epv_t;
extern codesets_v1_0_epv_t codesets_v1_0_c_epv;
extern rpc_if_handle_t codesets_v1_0_c_ifspec;
extern rpc_if_handle_t codesets_v1_0_s_ifspec;

#ifdef __cplusplus
    }
#endif

#endif
