/* Generated from rpc.sams on 1994-09-21-01:11:20.000 */
/* Do not edit! */
/*
 * 
 * (c) Copyright 1991 OPEN SOFTWARE FOUNDATION, INC.
 * (c) Copyright 1991 HEWLETT-PACKARD COMPANY
 * (c) Copyright 1991 DIGITAL EQUIPMENT CORPORATION
 * To anyone who acknowledges that this file is provided "AS IS"
 * without any express or implied warranty:
 *                 permission to use, copy, modify, and distribute this
 * file for any purpose is hereby granted without fee, provided that
 * the above copyright notices and this notice appears in all source
 * code copies, and that none of the names of Open Software
 * Foundation, Inc., Hewlett-Packard Company, or Digital Equipment
 * Corporation be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior
 * permission.  Neither Open Software Foundation, Inc., Hewlett-
 * Packard Company, nor Digital Equipment Corporation makes any
 * representations about the suitability of this software for any
 * purpose.
 * 
 */
interface rpcsts
{
    const unsigned32 rpc_s_mod                       = 0x16c9a000;
    const unsigned32 rpc_s_op_rng_error              = 0x16c9a001;
    const unsigned32 rpc_s_cant_create_socket        = 0x16c9a002;
    const unsigned32 rpc_s_cant_bind_socket          = 0x16c9a003;
    const unsigned32 rpc_s_not_in_call               = 0x16c9a004;
    const unsigned32 rpc_s_no_port                   = 0x16c9a005;
    const unsigned32 rpc_s_wrong_boot_time           = 0x16c9a006;
    const unsigned32 rpc_s_too_many_sockets          = 0x16c9a007;
    const unsigned32 rpc_s_illegal_register          = 0x16c9a008;
    const unsigned32 rpc_s_cant_recv                 = 0x16c9a009;
    const unsigned32 rpc_s_bad_pkt                   = 0x16c9a00a;
    const unsigned32 rpc_s_unbound_handle            = 0x16c9a00b;
    const unsigned32 rpc_s_addr_in_use               = 0x16c9a00c;
    const unsigned32 rpc_s_in_args_too_big           = 0x16c9a00d;
    const unsigned32 rpc_s_string_too_long           = 0x16c9a00e;
    const unsigned32 rpc_s_too_many_objects          = 0x16c9a00f;
    const unsigned32 rpc_s_binding_has_no_auth       = 0x16c9a010;
    const unsigned32 rpc_s_unknown_authn_service     = 0x16c9a011;
    const unsigned32 rpc_s_no_memory                 = 0x16c9a012;
    const unsigned32 rpc_s_cant_nmalloc              = 0x16c9a013;
    const unsigned32 rpc_s_call_faulted              = 0x16c9a014;
    const unsigned32 rpc_s_call_failed               = 0x16c9a015;
    const unsigned32 rpc_s_comm_failure              = 0x16c9a016;
    const unsigned32 rpc_s_rpcd_comm_failure         = 0x16c9a017;
    const unsigned32 rpc_s_illegal_family_rebind     = 0x16c9a018;
    const unsigned32 rpc_s_invalid_handle            = 0x16c9a019;
    const unsigned32 rpc_s_coding_error              = 0x16c9a01a;
    const unsigned32 rpc_s_object_not_found          = 0x16c9a01b;
    const unsigned32 rpc_s_cthread_not_found         = 0x16c9a01c;
    const unsigned32 rpc_s_invalid_binding           = 0x16c9a01d;
    const unsigned32 rpc_s_already_registered        = 0x16c9a01e;
    const unsigned32 rpc_s_endpoint_not_found        = 0x16c9a01f;
    const unsigned32 rpc_s_invalid_rpc_protseq       = 0x16c9a020;
    const unsigned32 rpc_s_desc_not_registered       = 0x16c9a021;
    const unsigned32 rpc_s_already_listening         = 0x16c9a022;
    const unsigned32 rpc_s_no_protseqs               = 0x16c9a023;
    const unsigned32 rpc_s_no_protseqs_registered    = 0x16c9a024;
    const unsigned32 rpc_s_no_bindings               = 0x16c9a025;
    const unsigned32 rpc_s_max_descs_exceeded        = 0x16c9a026;
    const unsigned32 rpc_s_no_interfaces             = 0x16c9a027;
    const unsigned32 rpc_s_invalid_timeout           = 0x16c9a028;
    const unsigned32 rpc_s_cant_inq_socket           = 0x16c9a029;
    const unsigned32 rpc_s_invalid_naf_id            = 0x16c9a02a;
    const unsigned32 rpc_s_inval_net_addr            = 0x16c9a02b;
    const unsigned32 rpc_s_unknown_if                = 0x16c9a02c;
    const unsigned32 rpc_s_unsupported_type          = 0x16c9a02d;
    const unsigned32 rpc_s_invalid_call_opt          = 0x16c9a02e;
    const unsigned32 rpc_s_no_fault                  = 0x16c9a02f;
    const unsigned32 rpc_s_cancel_timeout            = 0x16c9a030;
    const unsigned32 rpc_s_call_cancelled            = 0x16c9a031;
    const unsigned32 rpc_s_invalid_call_handle       = 0x16c9a032;
    const unsigned32 rpc_s_cannot_alloc_assoc        = 0x16c9a033;
    const unsigned32 rpc_s_cannot_connect            = 0x16c9a034;
    const unsigned32 rpc_s_connection_aborted        = 0x16c9a035;
    const unsigned32 rpc_s_connection_closed         = 0x16c9a036;
    const unsigned32 rpc_s_cannot_accept             = 0x16c9a037;
    const unsigned32 rpc_s_assoc_grp_not_found       = 0x16c9a038;
    const unsigned32 rpc_s_stub_interface_error      = 0x16c9a039;
    const unsigned32 rpc_s_invalid_object            = 0x16c9a03a;
    const unsigned32 rpc_s_invalid_type              = 0x16c9a03b;
    const unsigned32 rpc_s_invalid_if_opnum          = 0x16c9a03c;
    const unsigned32 rpc_s_different_server_instance = 0x16c9a03d;
    const unsigned32 rpc_s_protocol_error            = 0x16c9a03e;
    const unsigned32 rpc_s_cant_recvmsg              = 0x16c9a03f;
    const unsigned32 rpc_s_invalid_string_binding    = 0x16c9a040;
    const unsigned32 rpc_s_connect_timed_out         = 0x16c9a041;
    const unsigned32 rpc_s_connect_rejected          = 0x16c9a042;
    const unsigned32 rpc_s_network_unreachable       = 0x16c9a043;
    const unsigned32 rpc_s_connect_no_resources      = 0x16c9a044;
    const unsigned32 rpc_s_rem_network_shutdown      = 0x16c9a045;
    const unsigned32 rpc_s_too_many_rem_connects     = 0x16c9a046;
    const unsigned32 rpc_s_no_rem_endpoint           = 0x16c9a047;
    const unsigned32 rpc_s_rem_host_down             = 0x16c9a048;
    const unsigned32 rpc_s_host_unreachable          = 0x16c9a049;
    const unsigned32 rpc_s_access_control_info_inv   = 0x16c9a04a;
    const unsigned32 rpc_s_loc_connect_aborted       = 0x16c9a04b;
    const unsigned32 rpc_s_connect_closed_by_rem     = 0x16c9a04c;
    const unsigned32 rpc_s_rem_host_crashed          = 0x16c9a04d;
    const unsigned32 rpc_s_invalid_endpoint_format   = 0x16c9a04e;
    const unsigned32 rpc_s_unknown_status_code       = 0x16c9a04f;
    const unsigned32 rpc_s_unknown_mgr_type          = 0x16c9a050;
    const unsigned32 rpc_s_assoc_creation_failed     = 0x16c9a051;
    const unsigned32 rpc_s_assoc_grp_max_exceeded    = 0x16c9a052;
    const unsigned32 rpc_s_assoc_grp_alloc_failed    = 0x16c9a053;
    const unsigned32 rpc_s_sm_invalid_state          = 0x16c9a054;
    const unsigned32 rpc_s_assoc_req_rejected        = 0x16c9a055;
    const unsigned32 rpc_s_assoc_shutdown            = 0x16c9a056;
    const unsigned32 rpc_s_tsyntaxes_unsupported     = 0x16c9a057;
    const unsigned32 rpc_s_context_id_not_found      = 0x16c9a058;
    const unsigned32 rpc_s_cant_listen_socket        = 0x16c9a059;
    const unsigned32 rpc_s_no_addrs                  = 0x16c9a05a;
    const unsigned32 rpc_s_cant_getpeername          = 0x16c9a05b;
    const unsigned32 rpc_s_cant_get_if_id            = 0x16c9a05c;
    const unsigned32 rpc_s_protseq_not_supported     = 0x16c9a05d;
    const unsigned32 rpc_s_call_orphaned             = 0x16c9a05e;
    const unsigned32 rpc_s_who_are_you_failed        = 0x16c9a05f;
    const unsigned32 rpc_s_unknown_reject            = 0x16c9a060;
    const unsigned32 rpc_s_type_already_registered   = 0x16c9a061;
    const unsigned32 rpc_s_stop_listening_disabled   = 0x16c9a062;
    const unsigned32 rpc_s_invalid_arg               = 0x16c9a063;
    const unsigned32 rpc_s_not_supported             = 0x16c9a064;
    const unsigned32 rpc_s_wrong_kind_of_binding     = 0x16c9a065;
    const unsigned32 rpc_s_authn_authz_mismatch      = 0x16c9a066;
    const unsigned32 rpc_s_call_queued               = 0x16c9a067;
    const unsigned32 rpc_s_cannot_set_nodelay        = 0x16c9a068;
    const unsigned32 rpc_s_not_rpc_tower             = 0x16c9a069;
    const unsigned32 rpc_s_invalid_rpc_protid        = 0x16c9a06a;
    const unsigned32 rpc_s_invalid_rpc_floor         = 0x16c9a06b;
    const unsigned32 rpc_s_call_timeout              = 0x16c9a06c;
    const unsigned32 rpc_s_mgmt_op_disallowed        = 0x16c9a06d;
    const unsigned32 rpc_s_manager_not_entered       = 0x16c9a06e;
    const unsigned32 rpc_s_calls_too_large_for_wk_ep = 0x16c9a06f;
    const unsigned32 rpc_s_server_too_busy           = 0x16c9a070;
    const unsigned32 rpc_s_prot_version_mismatch     = 0x16c9a071;
    const unsigned32 rpc_s_rpc_prot_version_mismatch = 0x16c9a072;
    const unsigned32 rpc_s_ss_no_import_cursor       = 0x16c9a073;
    const unsigned32 rpc_s_fault_addr_error          = 0x16c9a074;
    const unsigned32 rpc_s_fault_context_mismatch    = 0x16c9a075;
    const unsigned32 rpc_s_fault_fp_div_by_zero      = 0x16c9a076;
    const unsigned32 rpc_s_fault_fp_error            = 0x16c9a077;
    const unsigned32 rpc_s_fault_fp_overflow         = 0x16c9a078;
    const unsigned32 rpc_s_fault_fp_underflow        = 0x16c9a079;
    const unsigned32 rpc_s_fault_ill_inst            = 0x16c9a07a;
    const unsigned32 rpc_s_fault_int_div_by_zero     = 0x16c9a07b;
    const unsigned32 rpc_s_fault_int_overflow        = 0x16c9a07c;
    const unsigned32 rpc_s_fault_invalid_bound       = 0x16c9a07d;
    const unsigned32 rpc_s_fault_invalid_tag         = 0x16c9a07e;
    const unsigned32 rpc_s_fault_pipe_closed         = 0x16c9a07f;
    const unsigned32 rpc_s_fault_pipe_comm_error     = 0x16c9a080;
    const unsigned32 rpc_s_fault_pipe_discipline     = 0x16c9a081;
    const unsigned32 rpc_s_fault_pipe_empty          = 0x16c9a082;
    const unsigned32 rpc_s_fault_pipe_memory         = 0x16c9a083;
    const unsigned32 rpc_s_fault_pipe_order          = 0x16c9a084;
    const unsigned32 rpc_s_fault_remote_comm_failure = 0x16c9a085;
    const unsigned32 rpc_s_fault_remote_no_memory    = 0x16c9a086;
    const unsigned32 rpc_s_fault_unspec              = 0x16c9a087;
    const unsigned32 uuid_s_bad_version              = 0x16c9a088;
    const unsigned32 uuid_s_socket_failure           = 0x16c9a089;
    const unsigned32 uuid_s_getconf_failure          = 0x16c9a08a;
    const unsigned32 uuid_s_no_address               = 0x16c9a08b;
    const unsigned32 uuid_s_overrun                  = 0x16c9a08c;
    const unsigned32 uuid_s_internal_error           = 0x16c9a08d;
    const unsigned32 uuid_s_coding_error             = 0x16c9a08e;
    const unsigned32 uuid_s_invalid_string_uuid      = 0x16c9a08f;
    const unsigned32 uuid_s_no_memory                = 0x16c9a090;
    const unsigned32 rpc_s_no_more_entries           = 0x16c9a091;
    const unsigned32 rpc_s_unknown_ns_error          = 0x16c9a092;
    const unsigned32 rpc_s_name_service_unavailable  = 0x16c9a093;
    const unsigned32 rpc_s_incomplete_name           = 0x16c9a094;
    const unsigned32 rpc_s_group_not_found           = 0x16c9a095;
    const unsigned32 rpc_s_invalid_name_syntax       = 0x16c9a096;
    const unsigned32 rpc_s_no_more_members           = 0x16c9a097;
    const unsigned32 rpc_s_no_more_interfaces        = 0x16c9a098;
    const unsigned32 rpc_s_invalid_name_service      = 0x16c9a099;
    const unsigned32 rpc_s_no_name_mapping           = 0x16c9a09a;
    const unsigned32 rpc_s_profile_not_found         = 0x16c9a09b;
    const unsigned32 rpc_s_not_found                 = 0x16c9a09c;
    const unsigned32 rpc_s_no_updates                = 0x16c9a09d;
    const unsigned32 rpc_s_update_failed             = 0x16c9a09e;
    const unsigned32 rpc_s_no_match_exported         = 0x16c9a09f;
    const unsigned32 rpc_s_entry_not_found           = 0x16c9a0a0;
    const unsigned32 rpc_s_invalid_inquiry_context   = 0x16c9a0a1;
    const unsigned32 rpc_s_interface_not_found       = 0x16c9a0a2;
    const unsigned32 rpc_s_group_member_not_found    = 0x16c9a0a3;
    const unsigned32 rpc_s_entry_already_exists      = 0x16c9a0a4;
    const unsigned32 rpc_s_nsinit_failure            = 0x16c9a0a5;
    const unsigned32 rpc_s_unsupported_name_syntax   = 0x16c9a0a6;
    const unsigned32 rpc_s_no_more_elements          = 0x16c9a0a7;
    const unsigned32 rpc_s_no_ns_permission          = 0x16c9a0a8;
    const unsigned32 rpc_s_invalid_inquiry_type      = 0x16c9a0a9;
    const unsigned32 rpc_s_profile_element_not_found = 0x16c9a0aa;
    const unsigned32 rpc_s_profile_element_replaced  = 0x16c9a0ab;
    const unsigned32 rpc_s_import_already_done       = 0x16c9a0ac;
    const unsigned32 rpc_s_database_busy             = 0x16c9a0ad;
    const unsigned32 rpc_s_invalid_import_context    = 0x16c9a0ae;
    const unsigned32 rpc_s_uuid_set_not_found        = 0x16c9a0af;
    const unsigned32 rpc_s_uuid_member_not_found     = 0x16c9a0b0;
    const unsigned32 rpc_s_no_interfaces_exported    = 0x16c9a0b1;
    const unsigned32 rpc_s_tower_set_not_found       = 0x16c9a0b2;
    const unsigned32 rpc_s_tower_member_not_found    = 0x16c9a0b3;
    const unsigned32 rpc_s_obj_uuid_not_found        = 0x16c9a0b4;
    const unsigned32 rpc_s_no_more_bindings          = 0x16c9a0b5;
    const unsigned32 rpc_s_invalid_priority          = 0x16c9a0b6;
    const unsigned32 rpc_s_not_rpc_entry             = 0x16c9a0b7;
    const unsigned32 rpc_s_invalid_lookup_context    = 0x16c9a0b8;
    const unsigned32 rpc_s_binding_vector_full       = 0x16c9a0b9;
    const unsigned32 rpc_s_cycle_detected            = 0x16c9a0ba;
    const unsigned32 rpc_s_nothing_to_export         = 0x16c9a0bb;
    const unsigned32 rpc_s_nothing_to_unexport       = 0x16c9a0bc;
    const unsigned32 rpc_s_invalid_vers_option       = 0x16c9a0bd;
    const unsigned32 rpc_s_no_rpc_data               = 0x16c9a0be;
    const unsigned32 rpc_s_mbr_picked                = 0x16c9a0bf;
    const unsigned32 rpc_s_not_all_objs_unexported   = 0x16c9a0c0;
    const unsigned32 rpc_s_no_entry_name             = 0x16c9a0c1;
    const unsigned32 rpc_s_priority_group_done       = 0x16c9a0c2;
    const unsigned32 rpc_s_partial_results           = 0x16c9a0c3;
    const unsigned32 rpc_s_no_env_setup              = 0x16c9a0c4;
    const unsigned32 twr_s_unknown_sa                = 0x16c9a0c5;
    const unsigned32 twr_s_unknown_tower             = 0x16c9a0c6;
    const unsigned32 twr_s_not_implemented           = 0x16c9a0c7;
    const unsigned32 rpc_s_max_calls_too_small       = 0x16c9a0c8;
    const unsigned32 rpc_s_cthread_create_failed     = 0x16c9a0c9;
    const unsigned32 rpc_s_cthread_pool_exists       = 0x16c9a0ca;
    const unsigned32 rpc_s_cthread_no_such_pool      = 0x16c9a0cb;
    const unsigned32 rpc_s_cthread_invoke_disabled   = 0x16c9a0cc;
    const unsigned32 ept_s_cant_perform_op           = 0x16c9a0cd;
    const unsigned32 ept_s_no_memory                 = 0x16c9a0ce;
    const unsigned32 ept_s_database_invalid          = 0x16c9a0cf;
    const unsigned32 ept_s_cant_create               = 0x16c9a0d0;
    const unsigned32 ept_s_cant_access               = 0x16c9a0d1;
    const unsigned32 ept_s_database_already_open     = 0x16c9a0d2;
    const unsigned32 ept_s_invalid_entry             = 0x16c9a0d3;
    const unsigned32 ept_s_update_failed             = 0x16c9a0d4;
    const unsigned32 ept_s_invalid_context           = 0x16c9a0d5;
    const unsigned32 ept_s_not_registered            = 0x16c9a0d6;
    const unsigned32 ept_s_server_unavailable        = 0x16c9a0d7;
    const unsigned32 rpc_s_underspecified_name       = 0x16c9a0d8;
    const unsigned32 rpc_s_invalid_ns_handle         = 0x16c9a0d9;
    const unsigned32 rpc_s_unknown_error             = 0x16c9a0da;
    const unsigned32 rpc_s_ss_char_trans_open_fail   = 0x16c9a0db;
    const unsigned32 rpc_s_ss_char_trans_short_file  = 0x16c9a0dc;
    const unsigned32 rpc_s_ss_context_damaged        = 0x16c9a0dd;
    const unsigned32 rpc_s_ss_in_null_context        = 0x16c9a0de;
    const unsigned32 rpc_s_socket_failure            = 0x16c9a0df;
    const unsigned32 rpc_s_unsupported_protect_level = 0x16c9a0e0;
    const unsigned32 rpc_s_invalid_checksum          = 0x16c9a0e1;
    const unsigned32 rpc_s_invalid_credentials       = 0x16c9a0e2;
    const unsigned32 rpc_s_credentials_too_large     = 0x16c9a0e3;
    const unsigned32 rpc_s_call_id_not_found         = 0x16c9a0e4;
    const unsigned32 rpc_s_key_id_not_found          = 0x16c9a0e5;
    const unsigned32 rpc_s_auth_bad_integrity        = 0x16c9a0e6;
    const unsigned32 rpc_s_auth_tkt_expired          = 0x16c9a0e7;
    const unsigned32 rpc_s_auth_tkt_nyv              = 0x16c9a0e8;
    const unsigned32 rpc_s_auth_repeat               = 0x16c9a0e9;
    const unsigned32 rpc_s_auth_not_us               = 0x16c9a0ea;
    const unsigned32 rpc_s_auth_badmatch             = 0x16c9a0eb;
    const unsigned32 rpc_s_auth_skew                 = 0x16c9a0ec;
    const unsigned32 rpc_s_auth_badaddr              = 0x16c9a0ed;
    const unsigned32 rpc_s_auth_badversion           = 0x16c9a0ee;
    const unsigned32 rpc_s_auth_msg_type             = 0x16c9a0ef;
    const unsigned32 rpc_s_auth_modified             = 0x16c9a0f0;
    const unsigned32 rpc_s_auth_badorder             = 0x16c9a0f1;
    const unsigned32 rpc_s_auth_badkeyver            = 0x16c9a0f2;
    const unsigned32 rpc_s_auth_nokey                = 0x16c9a0f3;
    const unsigned32 rpc_s_auth_mut_fail             = 0x16c9a0f4;
    const unsigned32 rpc_s_auth_baddirection         = 0x16c9a0f5;
    const unsigned32 rpc_s_auth_method               = 0x16c9a0f6;
    const unsigned32 rpc_s_auth_badseq               = 0x16c9a0f7;
    const unsigned32 rpc_s_auth_inapp_cksum          = 0x16c9a0f8;
    const unsigned32 rpc_s_auth_field_toolong        = 0x16c9a0f9;
    const unsigned32 rpc_s_invalid_crc               = 0x16c9a0fa;
    const unsigned32 rpc_s_binding_incomplete        = 0x16c9a0fb;
    const unsigned32 rpc_s_key_func_not_allowed      = 0x16c9a0fc;
    const unsigned32 rpc_s_unknown_stub_rtl_if_vers  = 0x16c9a0fd;
    const unsigned32 rpc_s_unknown_ifspec_vers       = 0x16c9a0fe;
    const unsigned32 rpc_s_proto_unsupp_by_auth      = 0x16c9a0ff;
    const unsigned32 rpc_s_authn_challenge_malformed = 0x16c9a100;
    const unsigned32 rpc_s_protect_level_mismatch    = 0x16c9a101;
    const unsigned32 rpc_s_no_mepv                   = 0x16c9a102;
    const unsigned32 rpc_s_stub_protocol_error       = 0x16c9a103;
    const unsigned32 rpc_s_class_version_mismatch    = 0x16c9a104;
    const unsigned32 rpc_s_helper_not_running        = 0x16c9a105;
    const unsigned32 rpc_s_helper_short_read         = 0x16c9a106;
    const unsigned32 rpc_s_helper_catatonic          = 0x16c9a107;
    const unsigned32 rpc_s_helper_aborted            = 0x16c9a108;
    const unsigned32 rpc_s_not_in_kernel             = 0x16c9a109;
    const unsigned32 rpc_s_helper_wrong_user         = 0x16c9a10a;
    const unsigned32 rpc_s_helper_overflow           = 0x16c9a10b;
    const unsigned32 rpc_s_dg_need_way_auth          = 0x16c9a10c;
    const unsigned32 rpc_s_unsupported_auth_subtype  = 0x16c9a10d;
    const unsigned32 rpc_s_wrong_pickle_type         = 0x16c9a10e;
    const unsigned32 rpc_s_not_listening             = 0x16c9a10f;
    const unsigned32 rpc_s_ss_bad_buffer             = 0x16c9a110;
    const unsigned32 rpc_s_ss_bad_es_action          = 0x16c9a111;
    const unsigned32 rpc_s_ss_wrong_es_version       = 0x16c9a112;
    const unsigned32 rpc_s_fault_user_defined        = 0x16c9a113;
    const unsigned32 rpc_s_ss_incompatible_codesets  = 0x16c9a114;
    const unsigned32 rpc_s_tx_not_in_transaction     = 0x16c9a115;
    const unsigned32 rpc_s_tx_open_failed            = 0x16c9a116;
    const unsigned32 rpc_s_partial_credentials       = 0x16c9a117;
    const unsigned32 rpc_s_ss_invalid_codeset_tag    = 0x16c9a118;
    const unsigned32 rpc_s_mgmt_bad_type             = 0x16c9a119;
    const unsigned32 rpc_s_ss_invalid_char_input     = 0x16c9a11a;
    const unsigned32 rpc_s_ss_short_conv_buffer      = 0x16c9a11b;
    const unsigned32 rpc_s_ss_iconv_error            = 0x16c9a11c;
    const unsigned32 rpc_s_ss_no_compat_codeset      = 0x16c9a11d;
    const unsigned32 rpc_s_ss_no_compat_charsets     = 0x16c9a11e;
    const unsigned32 dce_cs_c_ok                     = 0x16c9a11f;
    const unsigned32 dce_cs_c_unknown                = 0x16c9a120;
    const unsigned32 dce_cs_c_notfound               = 0x16c9a121;
    const unsigned32 dce_cs_c_cannot_open_file       = 0x16c9a122;
    const unsigned32 dce_cs_c_cannot_read_file       = 0x16c9a123;
    const unsigned32 dce_cs_c_cannot_allocate_memory = 0x16c9a124;
    const unsigned32 rpc_s_ss_cleanup_failed         = 0x16c9a125;
    const unsigned32 rpc_svc_desc_general            = 0x16c9a126;
    const unsigned32 rpc_svc_desc_mutex              = 0x16c9a127;
    const unsigned32 rpc_svc_desc_xmit               = 0x16c9a128;
    const unsigned32 rpc_svc_desc_recv               = 0x16c9a129;
    const unsigned32 rpc_svc_desc_dg_state           = 0x16c9a12a;
    const unsigned32 rpc_svc_desc_cancel             = 0x16c9a12b;
    const unsigned32 rpc_svc_desc_orphan             = 0x16c9a12c;
    const unsigned32 rpc_svc_desc_cn_state           = 0x16c9a12d;
    const unsigned32 rpc_svc_desc_cn_pkt             = 0x16c9a12e;
    const unsigned32 rpc_svc_desc_pkt_quotas         = 0x16c9a12f;
    const unsigned32 rpc_svc_desc_auth               = 0x16c9a130;
    const unsigned32 rpc_svc_desc_source             = 0x16c9a131;
    const unsigned32 rpc_svc_desc_stats              = 0x16c9a132;
    const unsigned32 rpc_svc_desc_mem                = 0x16c9a133;
    const unsigned32 rpc_svc_desc_mem_type           = 0x16c9a134;
    const unsigned32 rpc_svc_desc_dg_pktlog          = 0x16c9a135;
    const unsigned32 rpc_svc_desc_thread_id          = 0x16c9a136;
    const unsigned32 rpc_svc_desc_timestamp          = 0x16c9a137;
    const unsigned32 rpc_svc_desc_cn_errors          = 0x16c9a138;
    const unsigned32 rpc_svc_desc_conv_thread        = 0x16c9a139;
    const unsigned32 rpc_svc_desc_pid                = 0x16c9a13a;
    const unsigned32 rpc_svc_desc_atfork             = 0x16c9a13b;
    const unsigned32 rpc_svc_desc_cma_thread         = 0x16c9a13c;
    const unsigned32 rpc_svc_desc_inherit            = 0x16c9a13d;
    const unsigned32 rpc_svc_desc_dg_sockets         = 0x16c9a13e;
    const unsigned32 rpc_svc_desc_timer              = 0x16c9a13f;
    const unsigned32 rpc_svc_desc_threads            = 0x16c9a140;
    const unsigned32 rpc_svc_desc_server_call        = 0x16c9a141;
    const unsigned32 rpc_svc_desc_nsi                = 0x16c9a142;
    const unsigned32 rpc_svc_desc_dg_pkt             = 0x16c9a143;
    const unsigned32 rpc_m_cn_ill_state_trans_sa     = 0x16c9a144;
    const unsigned32 rpc_m_cn_ill_state_trans_ca     = 0x16c9a145;
    const unsigned32 rpc_m_cn_ill_state_trans_sg     = 0x16c9a146;
    const unsigned32 rpc_m_cn_ill_state_trans_cg     = 0x16c9a147;
    const unsigned32 rpc_m_cn_ill_state_trans_sr     = 0x16c9a148;
    const unsigned32 rpc_m_cn_ill_state_trans_cr     = 0x16c9a149;
    const unsigned32 rpc_m_bad_pkt_type              = 0x16c9a14a;
    const unsigned32 rpc_m_prot_mismatch             = 0x16c9a14b;
    const unsigned32 rpc_m_frag_toobig               = 0x16c9a14c;
    const unsigned32 rpc_m_unsupp_stub_rtl_if        = 0x16c9a14d;
    const unsigned32 rpc_m_unhandled_callstate       = 0x16c9a14e;
    const unsigned32 rpc_m_call_failed               = 0x16c9a14f;
    const unsigned32 rpc_m_call_failed_no_status     = 0x16c9a150;
    const unsigned32 rpc_m_call_failed_errno         = 0x16c9a151;
    const unsigned32 rpc_m_call_failed_s             = 0x16c9a152;
    const unsigned32 rpc_m_call_failed_c             = 0x16c9a153;
    const unsigned32 rpc_m_errmsg_toobig             = 0x16c9a154;
    const unsigned32 rpc_m_invalid_srchattr          = 0x16c9a155;
    const unsigned32 rpc_m_nts_not_found             = 0x16c9a156;
    const unsigned32 rpc_m_invalid_accbytcnt         = 0x16c9a157;
    const unsigned32 rpc_m_pre_v2_ifspec             = 0x16c9a158;
    const unsigned32 rpc_m_unk_ifspec                = 0x16c9a159;
    const unsigned32 rpc_m_recvbuf_toosmall          = 0x16c9a15a;
    const unsigned32 rpc_m_unalign_authtrl           = 0x16c9a15b;
    const unsigned32 rpc_m_unexpected_exc            = 0x16c9a15c;
    const unsigned32 rpc_m_no_stub_data              = 0x16c9a15d;
    const unsigned32 rpc_m_eventlist_full            = 0x16c9a15e;
    const unsigned32 rpc_m_unk_sock_type             = 0x16c9a15f;
    const unsigned32 rpc_m_unimp_call                = 0x16c9a160;
    const unsigned32 rpc_m_invalid_seqnum            = 0x16c9a161;
    const unsigned32 rpc_m_cant_create_uuid          = 0x16c9a162;
    const unsigned32 rpc_m_pre_v2_ss                 = 0x16c9a163;
    const unsigned32 rpc_m_dgpkt_pool_corrupt        = 0x16c9a164;
    const unsigned32 rpc_m_dgpkt_bad_free            = 0x16c9a165;
    const unsigned32 rpc_m_lookaside_corrupt         = 0x16c9a166;
    const unsigned32 rpc_m_alloc_fail                = 0x16c9a167;
    const unsigned32 rpc_m_realloc_fail              = 0x16c9a168;
    const unsigned32 rpc_m_cant_open_file            = 0x16c9a169;
    const unsigned32 rpc_m_cant_read_addr            = 0x16c9a16a;
    const unsigned32 rpc_svc_desc_libidl             = 0x16c9a16b;
    const unsigned32 rpc_m_ctxrundown_nomem          = 0x16c9a16c;
    const unsigned32 rpc_m_ctxrundown_exc            = 0x16c9a16d;
    const unsigned32 rpc_s_fault_codeset_conv_error  = 0x16c9a16e;
}
