/* Generated by IDL compiler version OSF DCE T1.1.0-03 */
#ifndef twr_v0_0_included
#define twr_v0_0_included
#ifndef IDLBASE_H
#include <dce/idlbase.h>
#endif

#ifdef __cplusplus
    extern "C" {
#endif

#ifndef nbase_v0_0_included
#include <dce/nbase.h>
#endif
#ifndef lbase_v0_0_included
#include <dce/lbase.h>
#endif
#define twr_s_ok error_status_ok
extern void twr_ip_lower_flrs_from_sa(
    /* [in] */ unsigned32 transport_layer_protocol,
    /* [in] */ sockaddr_p_t sa,
    /* [out] */ twr_p_t *tower,
    /* [out] */ unsigned32 *status
);
extern void twr_dnet_lower_flrs_from_sa(
    /* [in] */ sockaddr_p_t sa,
    /* [out] */ twr_p_t *tower,
    /* [out] */ unsigned32 *status
);
extern void twr_osi_lower_flrs_from_sa(
    /* [in] */ sockaddr_p_t sa,
    /* [out] */ twr_p_t *tower,
    /* [out] */ unsigned32 *status
);
extern void twr_dds_lower_flrs_from_sa(
    /* [in] */ unsigned32 transport_layer_protocol,
    /* [in] */ sockaddr_p_t sa,
    /* [out] */ twr_p_t *tower,
    /* [out] */ unsigned32 *status
);
extern void twr_ip_lower_flrs_to_sa(
    /* [in] */ byte_p_t tower_octet_string,
    /* [out] */ sockaddr_p_t *sa,
    /* [out] */ unsigned32 *sa_len,
    /* [out] */ unsigned32 *status
);
extern void twr_dnet_lower_flrs_to_sa(
    /* [in] */ byte_p_t tower_octet_string,
    /* [out] */ sockaddr_p_t *sa,
    /* [out] */ unsigned32 *sa_len,
    /* [out] */ unsigned32 *status
);
extern void twr_osi_lower_flrs_to_sa(
    /* [in] */ byte_p_t tower_octet_string,
    /* [out] */ sockaddr_p_t *sa,
    /* [out] */ unsigned32 *sa_len,
    /* [out] */ unsigned32 *status
);
extern void twr_dds_lower_flrs_to_sa(
    /* [in] */ byte_p_t tower_octet_string,
    /* [out] */ sockaddr_p_t *sa,
    /* [out] */ unsigned32 *sa_len,
    /* [out] */ unsigned32 *status
);

#ifdef __cplusplus
    }
#endif

#endif
