/* Generated by IDL compiler version OSF DCE T1.1.0-03 */
#ifndef rpc_iovector_v0_0_included
#define rpc_iovector_v0_0_included
#ifndef IDLBASE_H
#include <dce/idlbase.h>
#endif

#ifdef __cplusplus
    extern "C" {
#endif

#ifndef nbase_v0_0_included
#include <dce/nbase.h>
#endif
#ifndef lbase_v0_0_included
#include <dce/lbase.h>
#endif
typedef void (*rpc_buff_dealloc_fn_t)(
    /* [in] */ byte_p_t buff_addr
);
typedef struct  {
rpc_buff_dealloc_fn_t buff_dealloc;
unsigned8 flags;
unsigned8 pad1;
unsigned8 pad2;
unsigned8 pad3;
byte_p_t buff_addr;
unsigned32 buff_len;
byte_p_t data_addr;
unsigned32 data_len;
} rpc_iovector_elt_t;
typedef rpc_iovector_elt_t *rpc_iovector_elt_p_t;
#define rpc_c_iovector_elt_reused (1)
typedef struct  {
unsigned16 num_elt;
rpc_iovector_elt_t elt[1];
} rpc_iovector_t;
typedef rpc_iovector_t *rpc_iovector_p_t;
extern void rpc_iovector_dealloc(
    /* [in] */ rpc_iovector_p_t iovector,
    /* [out] */ unsigned32 *status
);

#ifdef __cplusplus
    }
#endif

#endif
