/* Generated by IDL compiler version OSF DCE T1.1.0-03 */
#ifndef nca_status_v0_0_included
#define nca_status_v0_0_included
#ifndef IDLBASE_H
#include <dce/idlbase.h>
#endif

#ifdef __cplusplus
    extern "C" {
#endif

#ifndef nbase_v0_0_included
#include <dce/nbase.h>
#endif
#define nca_s_comm_failure (469827585)
#define nca_s_op_rng_error (469827586)
#define nca_s_unk_if (469827587)
#define nca_s_wrong_boot_time (469827590)
#define nca_s_you_crashed (469827593)
#define nca_s_proto_error (469827595)
#define nca_s_out_args_too_big (469827603)
#define nca_s_server_too_busy (469827604)
#define nca_s_fault_string_too_long (469827605)
#define nca_s_unsupported_type (469827607)
#define nca_s_fault_int_div_by_zero (469762049)
#define nca_s_fault_addr_error (469762050)
#define nca_s_fault_fp_div_zero (469762051)
#define nca_s_fault_fp_underflow (469762052)
#define nca_s_fault_fp_overflow (469762053)
#define nca_s_fault_invalid_tag (469762054)
#define nca_s_fault_invalid_bound (469762055)
#define nca_s_rpc_version_mismatch (469762056)
#define nca_s_unspec_reject (469762057)
#define nca_s_bad_actid (469762058)
#define nca_s_who_are_you_failed (469762059)
#define nca_s_manager_not_entered (469762060)
#define nca_s_fault_cancel (469762061)
#define nca_s_fault_ill_inst (469762062)
#define nca_s_fault_fp_error (469762063)
#define nca_s_fault_int_overflow (469762064)
#define nca_s_fault_unspec (469762066)
#define nca_s_fault_remote_comm_failure (469762067)
#define nca_s_fault_pipe_empty (469762068)
#define nca_s_fault_pipe_closed (469762069)
#define nca_s_fault_pipe_order (469762070)
#define nca_s_fault_pipe_discipline (469762071)
#define nca_s_fault_pipe_comm_error (469762072)
#define nca_s_fault_pipe_memory (469762073)
#define nca_s_fault_context_mismatch (469762074)
#define nca_s_fault_remote_no_memory (469762075)
#define nca_s_invalid_pres_context_id (469762076)
#define nca_s_unsupported_authn_level (469762077)
#define nca_s_invalid_checksum (469762079)
#define nca_s_invalid_crc (469762080)
#define nca_s_fault_user_defined (469762081)
#define nca_s_fault_tx_open_failed (469762082)
#define nca_s_fault_codeset_conv_error (469762083)

#ifdef __cplusplus
    }
#endif

#endif
